package com.linkv.rtm_flutter_plugin;

import android.content.Context;
import android.nfc.Tag;
import android.util.Log;
import android.view.View;
import android.widget.FrameLayout;
import android.widget.RelativeLayout;

import androidx.annotation.NonNull;

import com.linkv.lvrtmsdk.LVRTMEngine;
import com.linkv.rtc.LVRTCEngine;
import com.linkv.rtc.render.LVDisplayView;
import com.linkv.rtcsdk.utils.LogUtils;

import java.util.Map;

import io.flutter.plugin.platform.PlatformView;

public class FlutterSurfaceView implements PlatformView, RtmFlutterPlugin.ViewEventListener {
    private FrameLayout frameLayout;
    final String TAG = "FlutterSurfaceView";
    String mUid;
    // 是否本地预览的view
    boolean mIsLocalPreview;
    int mViewId;
    boolean mIsStoped = false;

    FlutterSurfaceView(Context context, Map creationParams, int id) {
        LogUtils.d(TAG, "id  = " + id);

        mViewId = id;
        LVRTMEngine engine = RtmFlutterPlugin.mRtmEngine;
        String uid = (String) creationParams.get("uid");
        mIsLocalPreview = (boolean) creationParams.get("isLocalPreview");
        frameLayout = new FrameLayout(context);
        frameLayout.setLayoutParams(new FrameLayout.LayoutParams(RelativeLayout.LayoutParams.MATCH_PARENT, RelativeLayout.LayoutParams.MATCH_PARENT));

        if (mIsLocalPreview) {
            engine.startPreview(uid, frameLayout, false);
            mUid = uid;
        } else {
            engine.startPlayingStream(uid, frameLayout, true);
        }
    }

    @Override
    public View getView() {
        return frameLayout;
    }

    @Override
    public void dispose() {
        LogUtils.d(TAG, "dispose");
        if (mIsLocalPreview) {
//            RtmFlutterPlugin.mRtmEngine.stopPreview();
            RtmFlutterPlugin.removeListener(this);

            if (!mIsStoped) {
                onStopCapture(mViewId);
            }

        }
    }

    @Override
    public void onFlutterViewAttached(@NonNull View flutterView) {
        LogUtils.d(TAG, "onFlutterViewAttached");
        if (mIsLocalPreview) {
//            RtmFlutterPlugin.setEventListener(this);
            RtmFlutterPlugin.addListener(this);
        }
    }

    @Override
    public void onFlutterViewDetached() {
        LogUtils.d(TAG, "onFlutterViewDetached");
//        RtmFlutterPlugin.setEventListener(null);
        RtmFlutterPlugin.removeListener(this);
    }

    @Override
    public void onStartCapture(int viewId) {
        LVRTMEngine engine = RtmFlutterPlugin.mRtmEngine;
        // 本地预览View才有开启视频采集功能。
        if (engine != null && mIsLocalPreview && viewId == mViewId) {
            LogUtils.d(TAG, "onStartCapture viewId = "  + viewId);
            engine.startPreview(mUid, frameLayout, false);
            mIsStoped = false;
        }
    }

    @Override
    public void onStopCapture(int viewId) {
        LVRTMEngine engine = RtmFlutterPlugin.mRtmEngine;
        if (engine != null && mIsLocalPreview && viewId == mViewId) {
            LogUtils.d(TAG, "onStopCapture viewId = " + viewId);
            engine.stopCapture();
            mIsStoped = true;
        }

    }
}
