package com.linkv.rtm_flutter_plugin;


import android.app.Application;
import android.content.Context;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import androidx.annotation.NonNull;
import com.im.imcore.IMBridger;
import com.im.imlogic.IMMsg;
import com.im.imlogic.LVIMAudioMessage;
import com.im.imlogic.LVIMEnumDefine;
import com.im.imlogic.LVIMFileContent;
import com.im.imlogic.LVIMFileUtils;
import com.im.imlogic.LVIMImageMessage;
import com.im.imlogic.LVIMSDK;
import com.im.imlogic.LVIMSession;
import com.im.imlogic.LVIMVideoMessage;
import com.im.imlogic.LVPushContent;
import com.linkv.lvrtmsdk.LVRTMEngine;
import com.linkv.rtcsdk.LinkVRTCEngine;
import com.linkv.rtcsdk.bean.VideoQuality;
import com.linkv.rtcsdk.utils.LogUtils;
import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import org.json.JSONException;
import org.json.JSONObject;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_AUDIO;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_IMAGE;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_TEXT;
import static com.linkv.lvrtmsdk.LVRTMEngine.MessageSubType.IM_SUBTYPE_VIDEO;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_1080P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_180P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_270P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_360P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_540P;
import static com.linkv.rtcsdk.LinkVRTCEngine.VideoConfigLevel.VIDEO_CONFIG_720P;

/**
 * RtmFlutterPlugin
 */
public class RtmFlutterPlugin implements FlutterPlugin, MethodCallHandler {
//    private static ViewEventListener mViewEventListener;
    private static  ArrayList<ViewEventListener> previewListeners = new ArrayList<ViewEventListener>();
    private MethodChannel channel;
    static LVRTMEngine mRtmEngine;
    private String TAG = "RtmFlutterPlugin";
    private Context mContext;
    private boolean isTokenRequesting;
    private int ERROR_BUILD_MSG = 1001;
    Handler handler = new Handler(Looper.getMainLooper());
    private boolean isAllInit = false;

    private void runOnUiThread(Runnable runnable) {
        handler.post(runnable);
    }

    public RtmFlutterPlugin() {
    }

    @Override
    public void onAttachedToEngine(FlutterPluginBinding flutterPluginBinding) {
        LogUtils.d(TAG, "onAttachedToEngine");
        mContext = flutterPluginBinding.getApplicationContext();
        channel = new MethodChannel(flutterPluginBinding.getFlutterEngine().getDartExecutor(), "rtm_flutter_plugin");
        channel.setMethodCallHandler(this);
        flutterPluginBinding.getPlatformViewRegistry().registerViewFactory("surface", new FlutterSurfaceViewFactory(flutterPluginBinding.getBinaryMessenger()));
    }


    @Override
    public void onMethodCall(MethodCall call, @NonNull Result result) {

        LogUtils.d(TAG, "onMethodCall  : " + call.method);
        if (call.method.equals("getSdkVersion")) {
            result.success(mRtmEngine.getSdkVersion());
        } else if (call.method.equals("initRtcAndImSDK")) {
            initRtcAndImSDK(call, result);
        } else if (call.method.equals("initRtcSDK")) {
            initRtcSDK(call, result);
        } else if (call.method.equals("initImSDK")) {
            initImSDK(call, result);
        } else if (call.method.equals("loginUser")) {
            loginUser(call, result);
        } else if (call.method.equals("requestDebugToken")) {
            requestDebugToken(call, result);
        } else if (call.method.equals("setIMToken")) {
            setIMToken(call, result);
        } else if (call.method.equals("setLogOpen")) {
            setLogOpen(call);
        } else if (call.method.equals("sendPrivateMessage")) {
            sendPrivateMessage(call, result);
        } else if (call.method.equals("sendRoomMessage")) {
            sendRoomMessage(call, result);
        } else if (call.method.equals("uploadPushToken")) {
            uploadPushToken(call, result);
        } else if (call.method.equals("unInitRtcAndImSDK")) {
            unInitRtcAndImSDK();
        } else if (call.method.equals("unInitRtcSDK")) {
            unInitRtcSDK();
        } else if (call.method.equals("unInitImSDK")) {
            unInitImSDK();
        } else if (call.method.equals("loginRoom")) {
            loginRoom(call, result);
        } else if (call.method.equals("logoutRoom")) {
            logoutRoom(result);
        } else if (call.method.equals("logoutUser")) {
            logoutUser(result);
        } else if (call.method.equals("chatroomBlackUser")) {
            chatroomBlackUser(call, result);
        } else if (call.method.equals("chatroomGagUser")) {
            chatroomGagUser(call, result);
        } else if (call.method.equals("chatroomData")) {
            chatroomData(call, result);
        } else if (call.method.equals("chatroomUserData")) {
            chatroomUserData(call, result);
        } else if (call.method.equals("setPublishQualityMonitorCycle")) {
            setPublishQualityMonitorCycle(call, result);
        } else if (call.method.equals("startPublishing")) {
            startPublishing(call, result);
        } else if (call.method.equals("stopPublishStream")) {
            stopPublishStream(result);
        } else if (call.method.equals("startPlayingStream")) {
            startPlayingStream(call, result);
        } else if (call.method.equals("stopPlayingStream")) {
            stopPlayingStream(call, result);
        } else if (call.method.equals("setPlayQualityMonitor")) {
            setPlayQualityMonitor(call, result);
        } else if (call.method.equals("switchCamera")) {
            switchCamera(call);
        } else if (call.method.equals("setAVConfig")) {
            setAVConfig(call, result);
        } else if (call.method.equals("setMuteOutput")) {
            setMuteOutput(call);
        } else if (call.method.equals("startCapture")) {
            startCapture(call, result);
        } else if (call.method.equals("stopCapture")) {
            stopCapture(call, result);
        } else if (call.method.equals("isIMAuthed")) {
            isIMAuthed(result);
        } else if (call.method.equals("isIMLoginSucceed")) {
            isIMLoginSucceed(result);
        } else if (call.method.equals("setBeautyLevel")) {
            setBeautyLevel(call, result);
        } else if (call.method.equals("sendMessage")) {
            sendMessage(call, result);
        } else if (call.method.equals("updatePrivateMessageBlack")) {
            updatePrivateMessageBlack(call, result);
        } else if (call.method.equals("getPrivateMessageBlack")) {
            getPrivateMessageBlack(call, result);
        } else if (call.method.equals("sendEventMessage")) {
            sendEventMessage(call, result);
        } else if (call.method.equals("deleteLocalPrivateSession")) {
            deleteLocalPrivateSession(call, result);
        } else if (call.method.equals("clearSessionUnreadMsg")) {
            clearSessionUnreadMsg(call, result);
        } else if (call.method.equals("clearAllUnreadMsg")) {
            clearAllUnreadMsg(call, result);
        } else if (call.method.equals("updateMsgAsReadByStime")) {
            updateMsgAsReadByStime(call, result);
        } else if (call.method.equals("queryLocalPrivateHistoryMessage")) {
            queryLocalPrivateHistoryMessage(call, result);
        } else if (call.method.equals("querySessionList")) {
            querySessionList(call, result);
        } else if (call.method.equals("queryAllSessionList")) {
            queryAllSessionList(call, result);
        }  else if (call.method.equals("setPrivateDBStorageMax")) {
            setPrivateDBStorageMax(call,result);
        } else if (call.method.equals("queryUnReadMessageNum")) {
            queryUnReadMessageNum(call,result);
        } else if (call.method.equals("sendPrivateMessageReadedAck")) {
            sendPrivateMessageReadedAck(call,result);
        } else if (call.method.equals("localPathWithFileContent")) {
            localPathWithFileContent(call,result);
        } else if (call.method.equals("downloadFile")) {
            downloadFile(call,result);
        } else if (call.method.equals("sendFileMessageRequest")) {
            sendFileMessageRequest(call,result);
        } else if (call.method.equals("sendAudioMessageRequest")) {
            sendAudioMessageRequest(call,result);
        } else if (call.method.equals("downloadAudio")) {
            downloadAudio(call,result);
        } else if (call.method.equals("sendImageMessageRequest")) {
            sendImageMessageRequest(call,result);
        } else if (call.method.equals("downloadThumbnailImage")) {
            downloadThumbnailImage(call,result);
        } else if (call.method.equals("downloadImage")) {
            downloadImage(call,result);
        } else if (call.method.equals("sendVideoMessageRequest")) {
            sendVideoMessageRequest(call,result);
        } else if (call.method.equals("transVideo")) {
            transVideo(call,result);
        } else if (call.method.equals("downloadVideoThumbnailImage")) {
            downloadVideoThumbnailImage(call,result);
        } else if (call.method.equals("downloadVideoImage")) {
            downloadVideoImage(call,result);
        } else if (call.method.equals("downloadVideo")) {
            downloadVideo(call,result);
        }else if (call.method.equals("queryRemoteSessionMessage")) {
            queryRemoteSessionMessage(call,result);
        } else {
            result.notImplemented();
        }
    }

    /// 拉取远程某个会话的历史消息
    private void queryRemoteSessionMessage(MethodCall call, Result result) {
        String tuid = call.argument("tuid");
        int sequence = call.argument("sequence");
        int size = call.argument("size");
        int type = call.argument("type");

        LVIMSDK.sharedInstance().getMsgManager().queryRemoteSessionMessage(tuid, sequence, size, type, new IMBridger.IMQueryRemoteSessionMessageListener() {
            @Override
            public void onIMQueryRemoteSessionListCallback(int ecode, int rcode, int rstatus, List<IMMsg> list) {
                Map map = new HashMap();
                map.put("ecode", ecode);
                map.put("rcode", rcode);
                List array = new ArrayList();
                if (list != null && list.size() > 0) {
                    for (int i = 0; i < list.toArray().length; i++) {
                        array.add(msgToDict(list.get(i)));
                    }

                }
                map.put("messages", array != null ? array : new ArrayList());
                result.success(map);
            }
        });
    }

    /**
     * 下载文件方法
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadFile(MethodCall call, Result result) {
        String key = call.argument("key");
        int filetype = call.argument("filetype");
        String pathExtension = call.argument("pathExtension");
        LVIMFileContent content = new LVIMFileContent(key, filetype,pathExtension);
        content.downloadFile(new IMBridger.IMDownloadLintener() {
            @Override
            public void onComplete(int code) {
                result.success(code);
            }
        });
    }

    /**
     * 上传文件方法
     *
     * 成功返回0, 否则返回非0
     */
    private void sendFileMessageRequest(MethodCall call, Result result) {
        String path = call.argument("path");
//        String path = "/storage/emulated/0/charles.crt";
        int filetype = call.argument("filetype");
        LVIMFileContent content = new LVIMFileContent(path,filetype);
        content.uploadFileWithProgress(new IMBridger.IMUploadLintener() {
            @Override
            public void onProgress(double progress) {

            }

            @Override
            public void onComplete(int code) {
                result.success(code);
            }
        });
    }

    /**
     * 发送音视频数据
     *
     * 成功返回0, 否则返回非0
     */
    private void sendAudioMessageRequest(MethodCall call, Result result) {
        String uid = call.argument("uid");
        String tid = call.argument("tid");
        String title = call.argument("title");
        String body = call.argument("body");
        String audioPath = call.argument("audioPath");
//        String audioPath = "/storage/emulated/0/audio/20211026_173741.m4a";
        boolean isGroup = call.argument("isGroup");

        LVIMAudioMessage audioMessage = new LVIMAudioMessage();
        audioMessage.needSaveToDB = true;

        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.title = title;
        lvPushContent.body = body;

        if (isGroup) {
            audioMessage = LVIMAudioMessage.buildAudioGroupRequest(tid, "AUDIO", audioPath, "AUDIO", lvPushContent);
        } else {
            audioMessage = LVIMAudioMessage.buildAudioPrivateRequest(tid, "AUDIO", audioPath, "AUDIO", lvPushContent);
        }

        LVIMAudioMessage finalAudioMessage = audioMessage;
        audioMessage.uploadFiles(new IMBridger.IMUploadLintener() {
            @Override
            public void onComplete(int code) {
                _sendMediaMessages(finalAudioMessage,code,result);
            }
            @Override
            public void onProgress(double progress) {
            }
        });
    }

    /**
     * 下载音视频文件
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadAudio(MethodCall call, Result result) {
        LVIMAudioMessage audioMessage = (LVIMAudioMessage) LVIMAudioMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (audioMessage.subType == LVIMEnumDefine.IM_SUBTYPE_AUDIO) {
            if (audioMessage.audioFile.localPath == null) {
                audioMessage.downloadAudio(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(audioMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(audioMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }
    }

    /**
     * 发送图片数据
     *
     * 成功返回0, 否则返回非0
     */
    private void sendImageMessageRequest(MethodCall call, Result result) {
        String uid = call.argument("uid");
        String tid = call.argument("tid");
        String title = call.argument("title");
        String body = call.argument("body");
        String imagePath = call.argument("imagePath");
        boolean isGroup = call.argument("isGroup");

        LVIMImageMessage imageMessage = new LVIMImageMessage();
        imageMessage.needSaveToDB = true;

        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.title = title;
        lvPushContent.body = body;

        if (isGroup) {
            imageMessage = LVIMImageMessage.buildImageGroupRequest(tid, "IMAGE", imagePath, false,"IMAGE", LVIMEnumDefine.LVIM_IMAGE_FILE_TYPE_JPEG, lvPushContent);
        } else {
            imageMessage = LVIMImageMessage.buildImagePrivateRequest(tid, "IMAGE", imagePath, false,"IMAGE", LVIMEnumDefine.LVIM_IMAGE_FILE_TYPE_JPEG, lvPushContent);
        }

        LVIMImageMessage finalImageMessage = imageMessage;
        imageMessage.uploadFiles(new IMBridger.IMUploadLintener() {
            @Override
            public void onComplete(int code) {
                _sendMediaMessages(finalImageMessage,code,result);
            }
            @Override
            public void onProgress(double progress) {
            }
        });

    }

    /**
     * 下载缩略图片数据
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadThumbnailImage(MethodCall call, Result result) {
        LVIMImageMessage imageMessage = (LVIMImageMessage) LVIMImageMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (imageMessage.subType == LVIMEnumDefine.IM_SUBTYPE_IMAGE) {
            if (imageMessage.imageFile.localPath == null) {
                imageMessage.downloadThumbnailImage(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(imageMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(imageMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }
    }

    /**
     * 下载图片数据
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadImage(MethodCall call, Result result) {
        LVIMImageMessage imageMessage = (LVIMImageMessage) LVIMImageMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (imageMessage.subType == LVIMEnumDefine.IM_SUBTYPE_IMAGE) {
            if (imageMessage.imageFile.localPath == null) {
                imageMessage.downloadImage(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(imageMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(imageMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }
    }

    /**
     * 发送视频数据
     *
     * 成功返回0, 否则返回非0
     */
    private void sendVideoMessageRequest(MethodCall call, Result result) {
        String uid = call.argument("uid");
        String tid = call.argument("tid");
        String title = call.argument("title");
        String body = call.argument("body");
        String filePath = call.argument("filePath");
        boolean isGroup = call.argument("isGroup");

        LVIMVideoMessage videoMessage = new LVIMVideoMessage();
        videoMessage.needSaveToDB = true;

        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.title = title;
        lvPushContent.body = body;

        if (isGroup) {
            videoMessage = LVIMVideoMessage.buildVideoGroupRequest(tid, "VIDEO", filePath, "VIDEO", lvPushContent);
        } else {
            videoMessage = LVIMVideoMessage.buildVideoPrivateRequest(tid, "VIDEO", filePath, "VIDEO", lvPushContent);
        }

        LVIMVideoMessage finalVideoMessage = videoMessage;
        videoMessage.uploadFiles(new IMBridger.IMUploadLintener() {
            @Override
            public void onComplete(int code) {
               _sendMediaMessages(finalVideoMessage,code,result);
            }
            @Override
            public void onProgress(double progress) {
                System.out.println(progress);
            }
        });

    }

    /**
     * 转换视频数据
     *
     * 成功返回0, 否则返回非0
     */
    private void transVideo(MethodCall call, Result result) {
        String uid = call.argument("uid");
        String tid = call.argument("tid");
        String title = call.argument("title");
        String body = call.argument("body");
        String filePath = call.argument("filePath");
        boolean isGroup = call.argument("isGroup");

        LVIMVideoMessage videoMessage = new LVIMVideoMessage();
        videoMessage.needSaveToDB = true;

        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.title = title;
        lvPushContent.body = body;

        if (isGroup) {
            videoMessage = LVIMVideoMessage.buildVideoGroupRequest(tid, "VIDEO", filePath, "VIDEO", lvPushContent);
        } else {
            videoMessage = LVIMVideoMessage.buildVideoPrivateRequest(tid, "VIDEO", filePath, "VIDEO", lvPushContent);
        }

        videoMessage.transVideo(new IMBridger.IMTransVideoLintener() {
            @Override
            public void onComplete(int code,String newPath) {
                result.success(code);
            }
        });

    }

    /**
     * 下载视频缩略图
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadVideoThumbnailImage(MethodCall call, Result result) {
        LVIMVideoMessage videoMessage = (LVIMVideoMessage) LVIMVideoMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (videoMessage.subType == LVIMEnumDefine.IM_SUBTYPE_VIDEO) {
            if (videoMessage.imageFile.localPath == null) {
                videoMessage.downloadThumbnailImage(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(videoMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(videoMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }

    }

    /**
     * 下载视频
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadVideo(MethodCall call, Result result) {
        LVIMVideoMessage videoMessage = (LVIMVideoMessage) LVIMVideoMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (videoMessage.subType == LVIMEnumDefine.IM_SUBTYPE_VIDEO) {
            if (videoMessage.imageFile.localPath == null) {
                videoMessage.downloadVideo(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(videoMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(videoMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }

    }

    /**
     * 下载视频首帧图片
     *
     * 成功返回0, 否则返回非0
     */
    private void downloadVideoImage(MethodCall call, Result result) {
        LVIMVideoMessage videoMessage = (LVIMVideoMessage) LVIMVideoMessage.fromFlutterJsonObject(removeNull(call.arguments()));
        if (videoMessage.subType == LVIMEnumDefine.IM_SUBTYPE_VIDEO) {
            if (videoMessage.imageFile.localPath == null) {
                videoMessage.downloadImage(new IMBridger.IMDownloadLintener() {
                    @Override
                    public void onComplete(int code) {
                        final Map<String, Object> map = msgToDict(videoMessage);
                        map.put("eCode", code);
                        result.success(map);
                    }
                });
            }else{
                final Map<String, Object> map = msgToDict(videoMessage);
                map.put("eCode", 1);
                result.success(map);
            }
        }

    }

    /**
     * 获取本地路径
     *
     * 返回本地路径
     */
    private void localPathWithFileContent(MethodCall call, Result result) {
        String key = call.argument("key");
        int filetype = call.argument("filetype");
        String pathExtension = call.argument("pathExtension");
        LVIMFileContent lvimFileContent = new LVIMFileContent(key, filetype, pathExtension);
        String localPath = LVIMFileUtils.localPathWithFileContent(lvimFileContent);
        result.success(localPath);
    }

    // 私信消息已读回执
    private void sendPrivateMessageReadedAck(MethodCall call, Result result) {
        List<Map> listData = call.argument("listMsg");
        String sessionId = call.argument("sessionId");
        List listMsg = new ArrayList<>();
        for(Map data : listData){
            listMsg.add(IMMsg.fromFlutterJsonObject(removeNull(data)));
        }
        LVIMSDK.sharedInstance().getMsgManager().sendPrivateMessageReadedAck(listMsg, sessionId, this, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object context) {
                result.success(ecode);
            }
        });
    }

    private void setPrivateDBStorageMax(MethodCall call, Result result) {
        int max = call.argument("max");
        if (max <= 0 ){
            result.success(-1);
            return;
        }
        int maxResult = LVIMSDK.sharedInstance().msgManager.setDBStorageMax(3,max);
        result.success(maxResult);
    }

    private void queryUnReadMessageNum(MethodCall call, Result result) {
        int unReadNum = LVIMSDK.sharedInstance().conversationManager.queryUnReadMessageNum();
        result.success(unReadNum);
    }

    private void sendMessage(MethodCall call, Result result) {
        int cmdType = call.argument("cmdType");
        String fromID = call.argument("fromID");
        int sub = call.argument("sub");
        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        long cmsgid = call.argument("cmsgid");
        String pushContent = call.argument("pushContent");

        IMMsg msg = new IMMsg();
        msg.cmdType = cmdType;
        msg.fromID = fromID;
        msg.subType = sub;
        msg.toID = tid;
        msg.setMsgType(type);
        msg.setMsgContent(content);
        msg.setPushContent(pushContent);
        msg.cmsgid = cmsgid;

        int rs = LVIMSDK.sharedInstance().sendMessage(msg, null, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = msgToDict(msg);
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                result.success(messageResultMap);
            }
        });

        if (rs != 0) {
            final Map<String, Object> map = new HashMap<>();
            map.put("eCode", rs);
            result.success(map);
        }
    }

    /**
     *
     * 用户私信黑名单管理
     * uid 被拉黑的用户ID
     * isAdd true为添加,false为删除
     * 成功返回0, 否则返回非0
     */
    private void updatePrivateMessageBlack(MethodCall call, Result result) {
        String uid = call.argument("uid");
        boolean isAdd = call.argument("isAdd");
        LVIMSDK.sharedInstance().updatePrivateMessageBlack(uid, isAdd, mContext, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object context) {
                result.success(ecode);
            }
        });
    }

    /**
     * 获取用户私信黑名单列表
     * index 分页下标,第一页传入0，例如,每页20条数据,第一页传0,第二页传20
     * 成功返回0, 否则返回非0
     **/
    private void getPrivateMessageBlack(MethodCall call, Result result) {
        int index = call.argument("index");
        LVIMSDK.sharedInstance().getPrivateMessageBlack(index, mContext, new IMBridger.IMGroupListener<List<String>>() {
            @Override
            public void onGroupOprationCallback(int ecode, int rcode, List<String> response) {
                final Map<String, Object> map = new HashMap<>();
                map.put("eCode", ecode);
                map.put("response", response);
                result.success(map);
            }
        });
    }

    private void clearSessionUnreadMsg(MethodCall call, Result result) {
        String uid = call.argument("uid");
        int type = call.argument("type");
        int num = LVIMSDK.sharedInstance().conversationManager.clearSessionUnreadMsg(uid, type);
        result.success(num);
    }

    /// 获取消息会话列表
    private void querySessionList(MethodCall call, Result result) {
        List<LVIMSession> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        int type = call.argument("type");
        int num = LVIMSDK.sharedInstance().conversationManager.querySessionList(list, type);
        resultMap.put("result", num);

        List<Map<String, Object>> resultList = new ArrayList<>();
        for (LVIMSession session : list) {
            Map<String, Object> messageResultMap = msgToDict(session.getLastMsg());
            messageResultMap.put("unreadCount", session.getUnreadCount());
            resultList.add(messageResultMap);
        }
        resultMap.put("sessions", resultList);
        result.success(resultMap);
    }

    /**
     * @brief 查询本地数据库中全部的会话列表
     * sessions 结果集数据
     * @return 成功返回结果数量, 失败返回<0的值
     **/
    private void queryAllSessionList(MethodCall call, Result result) {
        List<LVIMSession> list = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        int num = LVIMSDK.sharedInstance().getConversationManager().queryAllSessionList(list);
        resultMap.put("result", num);

        List<Map<String, Object>> resultList = new ArrayList<>();
        for (LVIMSession session : list) {
            Map<String, Object> messageResultMap = msgToDict(session.getLastMsg());
            messageResultMap.put("unreadCount", session.getUnreadCount());
            resultList.add(messageResultMap);
        }
        resultMap.put("sessions", resultList);
        result.success(resultMap);
    }


    private void queryLocalPrivateHistoryMessage(MethodCall call, Result result) {
        Map<String, Object> resultMap = new HashMap<>();
        ArrayList<IMMsg> msgList = new ArrayList<>();
        String uid = call.argument("uid");
        int dbid = call.argument("dbid");
        int limit = call.argument("limit");
        dbid = dbid == 0 ? Integer.MAX_VALUE : dbid;

        int num = LVIMSDK.sharedInstance().msgManager.queryLocalPrivateHistoryMessage(uid, dbid, limit, true, msgList);
        resultMap.put("result", num);

        List<Map<String, Object>> resultList = new ArrayList<>();
        for (IMMsg msg : msgList) {
            Map<String, Object> messageResultMap = msgToDict(msg);
            resultList.add(messageResultMap);
        }
        resultMap.put("msgs", resultList);
        result.success(resultMap);
    }


    private void updateMsgAsReadByStime(MethodCall call, Result result) {
        String uid = call.argument("uid");
        long stime = call.argument("stime");
        int type = call.argument("type");
        int num = LVIMSDK.sharedInstance().conversationManager.updateMsgAsReadByStime(uid, stime, type);
        result.success(num);
    }


    private void clearAllUnreadMsg(MethodCall call, Result result) {
        int type = call.argument("type");
        int num = LVIMSDK.sharedInstance().conversationManager.clearAllUnreadMsg(type);
        result.success(num);
    }

    private void deleteLocalPrivateSession(MethodCall call, Result result) {
        String uid = call.argument("uid");
        int num = LVIMSDK.sharedInstance().msgManager.deleteLocalPrivateHistoryMessage(uid);
        result.success(num);
    }

    private void sendEventMessage(MethodCall call, Result result) {

        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        String  pushTitle = call.argument("pushTitle");
        String  pushBody = call.argument("pushBody");
        String  pushExtra = call.argument("pushExtra");
        String  pushClickAction = call.argument("pushClickAction");
        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.setTitle(pushTitle);
        lvPushContent.setBody(pushBody);
        lvPushContent.setExtra(pushExtra);
        lvPushContent.setClick_action(pushClickAction);
        IMMsg msg = IMMsg.buildEventMessage(tid, type, content, lvPushContent, null, null, null, null);

        // 构建消息失败
        if (msg == null) {
            final Map<String, Object> map = new HashMap<>();
            map.put("eCode", ERROR_BUILD_MSG);
            result.success(map);
            return;
        }

        // 构建消息成功
        int rs = LVIMSDK.sharedInstance().sendMessage(msg, null, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = msgToDict(msg);
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                result.success(messageResultMap);
            }
        });

        // 发送消息失败
        if (rs != 0) {
            final Map<String, Object> map = new HashMap<>();
            map.put("eCode", rs);
            result.success(map);
        }
    }


    private void setIMToken(MethodCall call, Result result) {
        String userId = call.argument("userId");
        String imToken = call.argument("imToken");
        LVIMSDK.sharedInstance().setIMToken(userId, imToken);
    }

    private void requestDebugToken(MethodCall call, final Result result) {
        Log.d(TAG, "requestDebugToken， thread " + Thread.currentThread().getName());
        /**
         * 此方法仅在调试模式下有效，demo演示用。
         * IM token请通过自己应用的服务端向IM SDK服务器请求，再转发给客户端，然后调用setIMToken方法。
         */
        LVIMSDK.sharedInstance().requestDebugToken(new LVIMSDK.RequestDebugTokenListener() {
            @Override
            public void onSucceed(final String imToken) {
                Log.d(TAG, "获取IM  onSucceed: " + imToken + " thread " + Thread.currentThread().getName());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        final Map<String, Object> map = new HashMap<>();
                        map.put("code", 0);
                        map.put("result", imToken);
                        result.success(map);
                    }
                });
            }

            @Override
            public void onFailed(final Exception e) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        final Map<String, Object> map = new HashMap<>();
                        map.put("code", -1);
                        map.put("result", e.toString());
                        result.success(map);
                    }
                });
                Log.d(TAG, "获取IM token失败， onFailure: " + e);
            }
        });
    }

    private void initRtcAndImSDK(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "initRtcAndImSDK");
        isAllInit = false;
        String rtcAppID = call.argument("rtcAppID");
        String rtcAppKey = call.argument("rtcAppKey");
        String imAppID = call.argument("imAppID");
        String imAppKey = call.argument("imAppKey");
        int sdkType = call.argument("sdkType");
        boolean isDebugEnv = call.argument("isDebugEnv");
        boolean isInternationalEnv = call.argument("isInternationalEnv");
        final Map<String, Object> map = new HashMap<>();
        mRtmEngine = LVRTMEngine.createEngine((Application) mContext.getApplicationContext(), rtcAppID, rtcAppKey, imAppID, imAppKey, sdkType, isDebugEnv, isInternationalEnv, new LinkVRTCEngine.RtcInitHandler() {
            @Override
            public void onRtcInitResult(int resultCode) {
                map.put("rtcCode", resultCode);
                if (isAllInit) {
                    result.success(map);
                } else {
                    isAllInit = true;
                }

            }
        }, new LVRTMEngine.ImInitHandler() {
            @Override
            public void onImInitResult(int resultCode) {
                map.put("imCode", resultCode);
                if (isAllInit) {
                    result.success(map);
                } else {
                    isAllInit = true;
                }
            }
        });
        mRtmEngine.setIMAuthEventListener(imAuthEventListener);
        mRtmEngine.setGlobalReceiveMessageListener(new IMBridger.IMReceiveMessageListener() {
            @Override
            public boolean onIMReceiveMessageFilter(int cmdtype, int subtype, int diytype, int dataid, String fromid, String toid, String msgType, byte[] msgContent, int waitings, int packetSize, int waitLength, int bufferSize) {
                if (cmdtype == LVIMEnumDefine.IM_CMDTYPE_ROOM){
                    LogUtils.d(TAG, "全局消息监听 ，过滤掉房间消息。 = " + cmdtype);
                    return true;
                }
                return false;
            }

            @Override
            public int onIMReceiveMessageHandler(String owner, IMMsg msg, int waitings, int packetSize, int waitLength, int bufferSize) {
                LogUtils.d(TAG, "onIMReceiveMessageHandler getMsgContent = " + msg.getMsgContent());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        channel.invokeMethod("onIMMessageReceive", msgToDict(msg));
                    }
                });
                return 0;
            }

            @Override
            public void onIMMessageReadedAcks(long[] longs) {
                channel.invokeMethod("onIMMessageReadedAcks", longs);
            }
        });
    }

    private void initRtcSDK(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "initRtcSDK");
        String rtcAppID = call.argument("rtcAppID");
        String rtcAppKey = call.argument("rtcAppKey");
        int sdkType = call.argument("sdkType");
        boolean isDebugEnv = call.argument("isDebugEnv");
        boolean isInternationalEnv = call.argument("isInternationalEnv");
        mRtmEngine = LVRTMEngine.createEngine((Application) mContext.getApplicationContext(), rtcAppID, rtcAppKey, sdkType, isDebugEnv, isInternationalEnv, new LinkVRTCEngine.RtcInitHandler() {
            @Override
            public void onRtcInitResult(int resultCode) {
                result.success(resultCode);
            }
        });
    }

    private void initImSDK(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "initImSDK");
        String imAppID = call.argument("imAppID");
        String imAppKey = call.argument("imAppKey");
        boolean isDebugEnv = call.argument("isDebugEnv");
        mRtmEngine = LVRTMEngine.createEngine((Application) mContext.getApplicationContext(), imAppID, imAppKey, isDebugEnv, new LVRTMEngine.ImInitHandler() {
            @Override
            public void onImInitResult(int resultCode) {
                result.success(resultCode);
            }
        });
        mRtmEngine.setIMAuthEventListener(imAuthEventListener);
        mRtmEngine.setGlobalReceiveMessageListener(new IMBridger.IMReceiveMessageListener() {
            @Override
            public boolean onIMReceiveMessageFilter(int cmdtype, int subtype, int diytype, int dataid, String fromid, String toid, String msgType, byte[] msgContent, int waitings, int packetSize, int waitLength, int bufferSize) {
                if (cmdtype == LVIMEnumDefine.IM_CMDTYPE_ROOM){
                    LogUtils.d(TAG, "全局消息监听 ，过滤掉房间消息。 = " + cmdtype);
                    return true;
                }
                return false;
            }

            @Override
            public int onIMReceiveMessageHandler(String owner, IMMsg msg, int waitings, int packetSize, int waitLength, int bufferSize) {
                LogUtils.d(TAG, "onIMReceiveMessageHandler getMsgContent = " + msg.getMsgContent());
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        channel.invokeMethod("onIMMessageReceive", msgToDict(msg));
                    }
                });
                return 0;
            }

            @Override
            public void onIMMessageReadedAcks(long[] longs) {
                channel.invokeMethod("onIMMessageReadedAcks", longs);
            }
        });
    }

    private void loginUser(final MethodCall call, final MethodChannel.Result result) {
        String userId = call.argument("userId");
        int loginType = call.argument("loginType");
        int resultCode = mRtmEngine.loginUser(userId, loginType);
        result.success(resultCode);
    }

    // 设置是否打开日志
    private void setLogOpen(MethodCall call) {
        boolean isOpen = call.argument("isOpen");
        mRtmEngine.setLogOpen(isOpen);
    }

    /**
     * 推流参数配置
     *
     * @param videoConfigLevel 推流参数预设值
     */
    public void setVideoConfig(int videoConfigLevel) {
//        mRtmEngine.setVideoConfig(videoConfigLevel);
    }


    // 发送私信消息
    private void sendPrivateMessage(MethodCall call, final Result result) {
        int sub = call.argument("sub");
        String tid = call.argument("tid");
        String type = call.argument("type");
        String content = call.argument("content");
        String  pushTitle = call.argument("pushTitle");
        String  pushBody = call.argument("pushBody");
        String  pushExtra = call.argument("pushExtra");
        String  pushClickAction = call.argument("pushClickAction");
        LVPushContent lvPushContent = new LVPushContent();
        lvPushContent.setTitle(pushTitle);
        lvPushContent.setBody(pushBody);
        lvPushContent.setExtra(pushExtra);
        lvPushContent.setClick_action(pushClickAction);
        mRtmEngine.sendPrivateMessage(convertSubTypeFromInt(sub), tid, type, content,lvPushContent, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = msgToDict(msg);
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                result.success(messageResultMap);
            }
        });
    }


    private LVRTMEngine.MessageSubType convertSubTypeFromInt(int sub) {
        LVRTMEngine.MessageSubType type = IM_SUBTYPE_TEXT;
        switch (sub) {
            case 1:
                type = IM_SUBTYPE_IMAGE;
                break;
            case 2:
                type = IM_SUBTYPE_AUDIO;
                break;
            case 3:
                type = IM_SUBTYPE_VIDEO;
                break;
            default:
                type = IM_SUBTYPE_TEXT;
                break;
        }
        return type;
    }


    // 发送房间消息
    void sendRoomMessage(MethodCall call, final Result result) {
        String targetId = call.argument("targetId");
        String msgContent = call.argument("msgContent");
        String msgType = call.argument("msgType");
        LogUtils.d(TAG, "sendRoomMessage msgContent :" + msgContent);

        mRtmEngine.sendRoomMessage(targetId, msgContent, msgType, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = msgToDict(msg);
                messageResultMap.put("eCode", ecode);
                result.success(messageResultMap);
            }
        });
    }

    private void setBeautyLevel(final MethodCall call, final Result result) {
        int beauty = call.argument("beauty");
        double level_d = call.argument("level");
        BigDecimal decimal = new BigDecimal(level_d);
        float level = decimal.floatValue();
        switch (beauty) {
            case 0:
                mRtmEngine.setBeautyLevel(level);
                break;
            case 1:
                mRtmEngine.setBrightLevel(level);
                break;
            case 2:
                mRtmEngine.setToneLevel(level);
                break;
            default:
                break;
        }
    }

    private void switchCamera(final MethodCall call) {
        boolean isFrontCamera = call.argument("isFrontCamera");
        mRtmEngine.useFrontCamera(isFrontCamera);
    }

    private void setAVConfig(final MethodCall call, final Result result) {
        int config = call.argument("config");
        mRtmEngine.setVideoConfig(convertVideoConfigFromInt(config));

        Log.d(TAG, "setAVConfig config = " + config);
        result.success(true);
    }


    private LinkVRTCEngine.VideoConfigLevel convertVideoConfigFromInt(int config) {
        LinkVRTCEngine.VideoConfigLevel configLevel = VIDEO_CONFIG_360P;
        switch (config) {
            case 0:
                configLevel = VIDEO_CONFIG_180P;
                break;
            case 1:
                configLevel = VIDEO_CONFIG_270P;
                break;
            case 2:
                configLevel = VIDEO_CONFIG_360P;
                break;
            case 3:
                configLevel = VIDEO_CONFIG_540P;
                break;
            case 4:
                configLevel = VIDEO_CONFIG_720P;
                break;
            case 5:
                configLevel = VIDEO_CONFIG_1080P;
                break;
            default:
                break;
        }
        return configLevel;
    }


    private void setMuteOutput(final MethodCall call) {
        boolean isMute = call.argument("isMute");
        mRtmEngine.setMuteOutput(isMute);
    }

    static public void addListener(ViewEventListener listener) {
        previewListeners.add(listener);
    }

    static public void removeListener(ViewEventListener listener) {
        previewListeners.remove(listener);
    }

    /**
     * 打开摄像头数据
     */
    private void startCapture(final MethodCall call, final Result result) {
//        if (mViewEventListener != null) {
//            mViewEventListener.onStartCapture();
//        }
        int viewId = call.argument("viewId");
        for (ViewEventListener listener : previewListeners) {
            listener.onStartCapture(viewId);
        }

        result.success(true);
    }

    /**
     * 关闭摄像头数据
     */
    private void stopCapture(final MethodCall call, final Result result) {
        int viewId = call.argument("viewId");
        for (ViewEventListener listener : previewListeners) {
            listener.onStopCapture(viewId);
        }

        result.success(true);
    }


    private void isIMAuthed(final Result result) {
        result.success(mRtmEngine.isIMAuthed());
    }

    private void isIMLoginSucceed(final Result result) {
        result.success(mRtmEngine.isIMLoginSucceed());
    }

    private void setPlayQualityMonitor(final MethodCall call, final Result result) {
        Log.d(TAG, "setPlayQualityMonitor seconds = " + call.argument("seconds"));
//        mRtmEngine.setPlayQualityMonitorCycle((int) call.argument("seconds"));
        result.success(true);
    }

    private void stopPlayingStream(final MethodCall call, final Result result) {
        String streamId = call.argument("streamId");
        mRtmEngine.stopPlayingStream(streamId);
        result.success(true);
    }

    private void startPlayingStream(final MethodCall call, final Result result) {
        String streamId = call.argument("streamId");
        Log.d(TAG, "startPlayingStream streamId = " + streamId);
        boolean isZOrderMediaOverlay = call.argument("isZOrderMediaOverlay");
        mRtmEngine.startPlayingStream(streamId, null, isZOrderMediaOverlay);
        result.success(true);
    }

    private void stopPublishStream(final Result result) {
        mRtmEngine.stopPublishingStream();
        result.success(true);
    }

    private void logoutRoom(final Result result) {
        Log.d(TAG, "logout start");
        mRtmEngine.logoutRoom();
    }

    private void logoutUser(final Result result) {
        Log.d(TAG, "logoutUser start");
        LVIMSDK.sharedInstance().logout();
    }

    /**
     * @brief 直播间用户拉黑
     *
     * rid      房间ID
     * blackid  拉黑用户ID
     * isAdd    添加或者删除
     * @return 成功返回0, 否则返回非0
     */
    private void chatroomBlackUser(final MethodCall call, final MethodChannel.Result result) {
        String rid = call.argument("rid");
        String blackid = call.argument("blackid");
        boolean isAdd = call.argument("isAdd");
        LVIMSDK.sharedInstance().getChatroomManager().chatroomBlackUser(rid, blackid
                , isAdd, mContext, new IMBridger.IMSendMessageListener() {
                    @Override
                    public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object context) {
                        result.success(ecode);
                    }
                });
    }

    /**
     * @brief 直播间用户禁言
     *
     * rid      房间ID
     * gagid  禁言用户ID
     * time   禁言时间
     * isAdd    添加或者删除
     * @return 成功返回0, 否则返回非0
     */
    private void chatroomGagUser(final MethodCall call, final MethodChannel.Result result) {
        String rid = call.argument("rid");
        String gagid = call.argument("gagid");
        int time = call.argument("time");
        boolean isAdd = call.argument("isAdd");
        LVIMSDK.sharedInstance().getChatroomManager().chatroomGagUser(rid, gagid
                , time, isAdd, mContext, new IMBridger.IMSendMessageListener() {
                    @Override
                    public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object context) {
                        result.success(ecode);
                    }
                });
    }

    /**
     *
     * @brief 房间数据
     * rid 房间ID
     * opType 操作方式/增删改查
     * key key
     * value value
     * @return 成功返回0, 否则返回非0
     */
    private void chatroomData(final MethodCall call, final MethodChannel.Result result) {
        String rid = call.argument("rid");
        int opType = call.argument("opType");
        String key = call.argument("key");
        String value = call.argument("value");
        LVIMSDK.sharedInstance().getChatroomManager().chatroomData(rid, opType, key, value, mContext, new IMBridger.IMChatRoomListener() {
            @Override
            public void onChatRoomCallback(int ecode, int rcode, Map response) {
                Map<String, Object> messageResultMap = new HashMap<>();
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                messageResultMap.put("response", response);
                result.success(messageResultMap);
            }
        });
    }

    /**
     * @brief 房间用户数据
     * rid 房间ID
     * opType 操作方式/增删改查
     * key key
     * value value
     * @return 成功返回0, 否则返回非0
     */
    private void chatroomUserData(final MethodCall call, final MethodChannel.Result result) {
        String rid = call.argument("rid");
        int opType = call.argument("opType");
        String key = call.argument("key");
        String value = call.argument("value");
        LVIMSDK.sharedInstance().getChatroomManager().chatroomUserData(rid, opType, key, value, mContext, new IMBridger.IMChatRoomListener() {
            @Override
            public void onChatRoomCallback(int ecode, int rcode, Map response) {
                Map<String, Object> messageResultMap = new HashMap<>();
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                messageResultMap.put("response", response);
                result.success(messageResultMap);
            }
        });
    }

    LVRTMEngine.IRoomEventHandler mRoomEventHandler = new LVRTMEngine.IRoomEventHandler() {
        @Override
        public void onRemoteStreamEnd(String userId) {
            LogUtils.d(TAG, "onRemoteStreamEnd user = " + userId);
            final Map<String, Object> map = new HashMap<>();
            map.put("userId", userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteStreamEnd", map);
                }
            });
        }

        @Override
        public void onRemoteStreamAdd(String userId) {
            LogUtils.d(TAG, "onRemoteStreamAdd userId = " + userId);

            final Map<String, Object> map = new HashMap<>();
            map.put("userId", userId);

            Log.d(TAG, "onAddRemoter Android user_id = " + userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteStreamAdd", map);
                }
            });
        }

        @Override
        public void onRoomDisconnect(int errorCode, String roomID) {
            LogUtils.d(TAG, "onRoomDisconnect errorCode = " + errorCode);

            final Map<String, Object> map = new HashMap<>();
            map.put("errorCode", errorCode);
            map.put("roomID", roomID);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomDisconnect", map);
                }
            });

        }

        @Override
        public void onRoomConnected(String roomID) {
            LogUtils.d(TAG, "onRoomConnected roomID = " + roomID);
            final Map<String, Object> map = new HashMap<>();
            map.put("roomID", roomID);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomConnected", map);
                }
            });
        }

        @Override
        public void onRemoteQualityUpdate(String streamID, VideoQuality quality) {
            final Map<String, Object> map = new HashMap<>();
            map.put("streamID", streamID);
            map.put("videoSentPackets", quality.videoSentPackets);
            map.put("videoSentKBytes", quality.videoSentKBytes);
            map.put("videoFps", quality.videoFps);
            map.put("videoLostPackets", quality.videoLostPackets);
            map.put("videoBitrateBps", quality.videoBitrateBps);
            map.put("videoRtt", quality.videoRtt);
            map.put("videoLostPercent", quality.videoLostPercent);
            map.put("frameWidth", quality.frameWidth);
            map.put("frameHeight", quality.frameHeight);

            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRemoteQualityUpdate", map);
                }
            });
        }

        @Override
        public void onPublisherQualityUpdate(String streamID, VideoQuality quality) {
            final Map<String, Object> map = new HashMap<>();
            map.put("videoSentPackets", quality.videoSentPackets);
            map.put("videoSentKBytes", quality.videoSentKBytes);
            map.put("videoFps", quality.videoFps);
            map.put("videoLostPackets", quality.videoLostPackets);
            map.put("videoBitrateBps", quality.videoBitrateBps);
            map.put("videoRtt", quality.videoRtt);
            map.put("videoLostPercent", quality.videoLostPercent);
            map.put("frameWidth", quality.frameWidth);
            map.put("frameHeight", quality.frameHeight);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPublisherQualityUpdate", map);
                }
            });
        }

        @Override
        public void onVideoSizeChanged(String streamID, int width, int height) {
            final Map<String, Object> map = new HashMap<>();
            map.put("streamID", streamID);
            map.put("width", width);
            map.put("height", height);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onVideoSizeChanged", map);
                }
            });
        }

        @Override
        public void onRoomMessageReceive(IMMsg msg) {
            LogUtils.d(TAG, "onRoomMessageReceive getMsgContent = " + msg.getMsgContent());
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onRoomMessageReceive", msgToDict(msg));
                }
            });
        }

        @Override
        public void onPublishStateUpdate(int state) {
            LogUtils.d(TAG, "onPublishStateUpdate state = " + state);
            final Map<String, Object> map = new HashMap<>();
            map.put("state", state);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPublishStateUpdate", map);
                }
            });
        }

        @Override
        public void onPlayStateUpdate(int state, String userId) {
            LogUtils.d(TAG, "onPlayStateUpdate state = " + state + "  userId: " + userId);
            final Map<String, Object> map = new HashMap<>();
            map.put("state", state);
            map.put("streamId", userId);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onPlayStateUpdate", map);
                }
            });
        }
    };


    IMBridger.IMModuleEventListener imAuthEventListener = new IMBridger.IMModuleEventListener() {
        @Override
        public void onQueryIMToken() { // 请在此回调中请求并设置IM token。此回调会频繁调用，请注意加逻辑控制避免频繁请求。示例如下。
            final Map<String, Object> map = new HashMap<>();
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onQueryIMToken", map);
                }
            });

        }

        @Override
        public void onIMAuthFailed(String uid, String token, int ecode, int rcode, boolean isTokenExpired) {
            // IM token校验失败会回调此方法。同时也回调onQueryIMToken().
            Log.d(TAG, "IM token校验失败 错误码 = " + ecode);
            final Map<String, Object> map = new HashMap<>();
            map.put("uid", uid);
            map.put("token", token);
            map.put("ecode", ecode);
            map.put("rcode", rcode);
            map.put("isTokenExpired", isTokenExpired);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMAuthFailed", map);
                }
            });
        }

        @Override
        public void onIMAuthSucceed(String uid, String token, long unReadMsgSize) {
            // IM校验token通过
            Log.d(TAG, "IM token校验通过 uid = " + uid + "    token =" + token);
            final Map<String, Object> map = new HashMap<>();
            map.put("uid", uid);
            map.put("token", token);
            map.put("unReadMsgSize", unReadMsgSize);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMAuthSucceed", map);
                }
            });
        }

        @Override
        public void onIMTokenExpired(String uid, String token) {
            // IM token过期会回调此方法。同时也回调onQueryIMToken().
            Log.d(TAG, "IM token过期 uid = " + uid + "    token =" + token);
            final Map<String, Object> map = new HashMap<>();
            map.put("token", token);
            map.put("uid", uid);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMTokenExpired", map);
                }
            });
        }

        /**
         * 连接成功事件
         */
        @Override
        public void onIMConnected(Object o) {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMConnected", null);
                }
            });
        }

        /**
         * 连接断开事件
         */
        @Override
        public void onIMLosted() {
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMLosted", null);
                }
            });
        }

        @Override
        public void onIMUserKicked(String uid, int code) {
            final Map<String, Object> map = new HashMap<>();
            map.put("uid", uid);
            map.put("code", code);
            runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    channel.invokeMethod("onIMUserKicked", map);
                }
            });
        }
    };

    private void unInitRtcAndImSDK() {
        if (mRtmEngine != null)
            mRtmEngine.unInitRtcAndImSDK();
        mRtmEngine = null;
    }

    private void unInitRtcSDK() {
        if (mRtmEngine != null)
            mRtmEngine.unInitRtcSDK();
        mRtmEngine = null;
    }

    private void unInitImSDK() {
        if (mRtmEngine != null)
            mRtmEngine.unInitImSDK();
        mRtmEngine = null;
    }

    private void loginRoom(final MethodCall call, final MethodChannel.Result result) {
        String userId = (String) call.argument("userId");
        Log.d(TAG, "loginRoom " + userId);
        mRtmEngine.setRoomEventHandler(mRoomEventHandler);
        mRtmEngine.loginRoom(userId, (String) call.argument("roomId"), (boolean) call.argument("isHost"), (boolean) call.argument("isAudioOnly"));
    }

    private void setPublishQualityMonitorCycle(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "setPublishQualityMonitorCycle");
//        mRtmEngine.setPublishQualityMonitorCycle((int) call.argument("seconds"));
        result.success(true);
    }

    private void startPublishing(final MethodCall call, final MethodChannel.Result result) {
        Log.d(TAG, "startPublishing");
        mRtmEngine.startPublishing();
        result.success(true);
    }


    @Override
    public void onDetachedFromEngine(FlutterPluginBinding binding) {
        LogUtils.d(TAG, "onDetachedFromEngine");
        channel.setMethodCallHandler(null);
    }


//    static void setEventListener(ViewEventListener listener) {
//        mViewEventListener = listener;
//    }


    public interface ViewEventListener {
        void onStartCapture(int viewId);

        void onStopCapture(int viewId);
    }

    private void uploadPushToken(final MethodCall call, final MethodChannel.Result result) {
        String token = call.argument("token");
        if (TextUtils.isEmpty(LVIMSDK.sharedInstance().getUserID())) {
            Map<String, Object> map = new HashMap<>();
            map.put("eCode", -1);
            map.put("msg", "please login IM first");
            result.success(map);
        }
        LVIMSDK.sharedInstance().uploadPushToken(token, new IMBridger.IMUploadPushTokenListener() {
            @Override
            public void onIMUploadPushTokenCallback(int eCode, String msg) {
                runOnUiThread(new Runnable() {
                    @Override
                    public void run() {
                        Map<String, Object> map = new HashMap<>();
                        map.put("eCode", eCode);
                        map.put("msg", msg);
                        result.success(map);
                    }
                });
            }
        });
    }


    /// 内部发送媒体消息
    private void  _sendMediaMessages(IMMsg mediaMessage,int code,final MethodChannel.Result result){
        if (code != 0) {
            final Map<String, Object> map = msgToDict(mediaMessage);
            map.put("eCode", code);
            result.success(map);
            return;
        }

        int rs = LVIMSDK.sharedInstance().sendMessage(mediaMessage, null, new IMBridger.IMSendMessageListener() {
            @Override
            public void onIMSendMessageCallback(int ecode, int rcode, String tid, IMMsg msg, Object o) {
                Map<String, Object> messageResultMap = msgToDict(mediaMessage);
                messageResultMap.put("eCode", ecode);
                messageResultMap.put("rcode", rcode);
                result.success(messageResultMap);
            }
        });

        if (rs != 0) {
            final Map<String, Object> map = msgToDict(mediaMessage);
            map.put("eCode", rs);
            result.success(map);
        }
    }

    //删除null 对象
    private JSONObject removeNull(Object dict){
        HashMap oldMap = (HashMap) dict;
        JSONObject map = new JSONObject();
        for (Iterator i = oldMap.keySet().iterator(); i.hasNext(); ) {
            Object obj = i.next();
            if (oldMap.get(obj) != null) {
                String key = (String) obj;
                Object value = (Object) oldMap.get(obj);
                //System.out.println("key=" + key + " value=" + value);
                try {
                    map.put(key, value);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            } else {
                System.out.println(obj + "value 异常");// 循环输出key
            }
        }
        return map;
    }

    // 将IMMsg对象的部分字段转成Map /生成多媒体json
    private Map<String, Object> msgToDict(IMMsg msg) {
        final Map<String, Object> map = new HashMap<>();
        map.put("origin", msg.origin);
        map.put("cmdType", msg.cmdType);
        map.put("subType", msg.subType);
        map.put("fromID", msg.fromID);
        map.put("toID", msg.toID);
        map.put("msgContent", msg.msgContent);
        map.put("msgType", msg.extend1);
        map.put("extend2", msg.extend2);
        map.put("extend3", msg.extend3);
        map.put("extend4", msg.extend4);
        map.put("extend5", msg.extend5);
        map.put("extend6", msg.extend6);
        map.put("pushTitle", msg.pushTitle);
        map.put("cmsgid", msg.cmsgid);
        map.put("smsgid", msg.smsgid);
        map.put("dbid", msg.dbid);
        map.put("ctime", msg.ctime);
        map.put("stime", msg.stime);
        IMMsg.LVIM_MESSAGE_SEND_STATE send_state = msg.send_state;
        int value = 0;
        if (send_state != null) {
            value = send_state.value();
        }
        map.put("sendState", value);
        map.put("sequence", msg.sequence);

        if (msg.subType == LVIMEnumDefine.IM_SUBTYPE_AUDIO) {// 音频
            LVIMAudioMessage audioMsg = (LVIMAudioMessage)msg;
            map.put("needSaveToDB",1);
            map.put("audioData",audioMsg.audioData);
            map.put("duration",String.valueOf(audioMsg.duration));
            map.put("audioFile",audioMsg.audioFile.jsonObject());

            final Map<String, Object> audioFile = new HashMap<>();
            audioFile.put("key",audioMsg.audioFile.key);
            audioFile.put("localPath",audioMsg.audioFile.localPath);
            audioFile.put("pathExtension",audioMsg.audioFile.pathExtension);
            audioFile.put("fileLength",audioMsg.audioFile.fileLength);
            audioFile.put("filetype",audioMsg.audioFile.filetype);
            map.put("audioFile",audioFile);

        }else if (msg.subType == LVIMEnumDefine.IM_SUBTYPE_IMAGE){//图片
            LVIMImageMessage imageMsg = (LVIMImageMessage)msg;
            map.put("needSaveToDB",1);
            map.put("image","");
            map.put("imageSize",imageMsg.imageSize.toString());
            map.put("thumbnailImage","");
            map.put("thumbnailImageSize",imageMsg.thumbnailImageSize.toString());
            map.put("isOrigin",imageMsg.isOrigin);

            final Map<String, Object> imageFile = new HashMap<>();
            imageFile.put("key",imageMsg.imageFile.key);
            imageFile.put("localPath",imageMsg.imageFile.localPath);
            imageFile.put("pathExtension",imageMsg.imageFile.pathExtension);
            imageFile.put("fileLength",imageMsg.imageFile.fileLength);
            imageFile.put("filetype",imageMsg.imageFile.filetype);
            map.put("imageFile",imageFile);

            final Map<String, Object> thumbnailImageFile = new HashMap<>();
            thumbnailImageFile.put("key",imageMsg.thumbnailImageFile.key);
            thumbnailImageFile.put("localPath",imageMsg.thumbnailImageFile.localPath);
            thumbnailImageFile.put("pathExtension",imageMsg.thumbnailImageFile.pathExtension);
            thumbnailImageFile.put("fileLength",imageMsg.thumbnailImageFile.fileLength);
            thumbnailImageFile.put("filetype",imageMsg.thumbnailImageFile.filetype);
            map.put("thumbnailImageFile",thumbnailImageFile);

        }else if (msg.subType == LVIMEnumDefine.IM_SUBTYPE_VIDEO) {//视频
            LVIMVideoMessage videoMsg = (LVIMVideoMessage)msg;
            map.put("needSaveToDB",1);
            map.put("videoSize",videoMsg.videoSize.toString());
            map.put("duration",String.valueOf(videoMsg.duration));
            map.put("image","");
            map.put("imageSize",videoMsg.imageSize.toString());
            map.put("thumbnailImage","");
            map.put("thumbnailImageSize",videoMsg.thumbnailImageSize.toString());

            final Map<String, Object> videoFile = new HashMap<>();
            videoFile.put("key",videoMsg.videoFile.key);
            videoFile.put("localPath",videoMsg.videoFile.localPath);
            videoFile.put("pathExtension",videoMsg.videoFile.pathExtension);
            videoFile.put("fileLength",videoMsg.videoFile.fileLength);
            videoFile.put("filetype",videoMsg.videoFile.filetype);
            map.put("imageFile",videoFile);

            final Map<String, Object> imageFile = new HashMap<>();
            imageFile.put("key",videoMsg.imageFile.key);
            imageFile.put("localPath",videoMsg.imageFile.localPath);
            imageFile.put("pathExtension",videoMsg.imageFile.pathExtension);
            imageFile.put("fileLength",videoMsg.imageFile.fileLength);
            imageFile.put("filetype",videoMsg.imageFile.filetype);
            map.put("imageFile",imageFile);

            final Map<String, Object> thumbnailImageFile = new HashMap<>();
            thumbnailImageFile.put("key",videoMsg.thumbnailImageFile.key);
            thumbnailImageFile.put("localPath",videoMsg.thumbnailImageFile.localPath);
            thumbnailImageFile.put("pathExtension",videoMsg.thumbnailImageFile.pathExtension);
            thumbnailImageFile.put("fileLength",videoMsg.thumbnailImageFile.fileLength);
            thumbnailImageFile.put("filetype",videoMsg.thumbnailImageFile.filetype);
            map.put("thumbnailImageFile",thumbnailImageFile);

        }

        return map;
    }
}
