import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:fluttertoast/fluttertoast.dart';
import 'package:image_picker/image_picker.dart';
import 'package:linkv_rtc_im/bean/lv_im_audio_msg.dart';
import 'package:linkv_rtc_im/bean/lv_im_msg.dart';
import 'package:linkv_rtc_im/linkv_api_defines.dart';
import 'package:linkv_rtc_im/linkv_rtm_callback.dart';
import 'package:linkv_rtc_im/rtm_flutter_plugin.dart';

class MediaMsgModel {
  String title;
  int subType;// 0文本  1图片 2音频 3视频
  Map json;
  var msg;

  static MediaMsgModel init(title,subType, json,{var msg}) {
    MediaMsgModel model = MediaMsgModel();
    model.title = title;
    model.subType = subType;
    model.json = json;
    model.msg = msg;
    return model;
  }
}

class IMMediaMsgPage extends StatefulWidget {
  IMMediaMsgPage({Key key, @required this.userid, @required this.targetUid})
      : super(key: key);
  final String userid;
  final String targetUid;

  @override
  _IMMediaMsgPageState createState() => _IMMediaMsgPageState();
}

class _IMMediaMsgPageState extends State<IMMediaMsgPage>
    implements LVIMEventCallback {
  /*1、对象 */
  var TAG = "【IM媒体服务】";
  List<MediaMsgModel> _msgList = <MediaMsgModel>[];
  TextEditingController msgController = TextEditingController();
  FocusNode focusNode = FocusNode();
  final ImagePicker _picker = ImagePicker();

  /*2、initState */
  @override
  void initState() {
    super.initState();

    RtmFlutterPlugin.setIMEventCallback(this);


    RtmFlutterPlugin.queryRemoteSessionMessage(widget.targetUid, 0, 10, LVIM_MESSAGE_LVDTYPE.IM_MESSAGE_LVDTYPE_PRIVATE).then((result){

      List list = result['messages'];
      List newList = [];
      for (var msg in list){
        newList.add(SendMessageResult.convertFromMap(msg));
      }
      if (list.length != 0){
       for (SendMessageResult data in newList){
         print(TAG + "queryRemoteSessionMessage" + " result = ${data.msg.isReaded}");

         _msgList.add(MediaMsgModel.init(data.msg.msgContent, data.msg.subType, data.msg.toMap()));
       }
       setState(() {});
      }

    });
  }

  /*3、dispose */
  @override
  void dispose() {
    // TODO: implement dispose
    super.dispose();
  }

  /*4、build 核心渲染 */
  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("测试媒体文件消息"),
      ),
      body: GestureDetector(
        onTap: () {
          FocusScope.of(context).requestFocus(FocusNode());
        },
        child: Column(
          children: [
            Expanded(
              child: ListView.builder(
                itemBuilder: (ctx, index) {
                  return Column(
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      InkWell(
                        onTap: () {
                          if (_msgList[index].subType == 1){//图片
                            downloadImage(_msgList[index].json,index);
                          } else if (_msgList[index].subType == 2) { //音频
                            downloadAudio(_msgList[index].json,index);
                          } else if (_msgList[index].subType == 3) { //视频
                            downloadVideoImage(_msgList[index].json,index);
                          }
                        },
                        child: Container(
                          padding: const EdgeInsets.only(left: 10, right: 10),
                          height: 50,
                          alignment: Alignment.centerLeft,
                          child: (_msgList[index].msg !=null) ? Row(children: [Image.file(File(_msgList[index]?.msg?.msg?.imageFile["localPath"])),Text(_msgList[index].title)],) :Text(_msgList[index].title),
                        ),
                      ),
                      Container(height: 1, color: Colors.grey),
                    ],
                  );
                },
                itemCount: _msgList.length,
              ),
            ),
            _toolBarWidget(),
          ],
        ),
      ),
    );
  }

  //发送消息
  void _sendMessageAction() {
    FocusScope.of(context).requestFocus(FocusNode());

    if (msgController.text.length == 0) {
      Fluttertoast.showToast(msg: "please msg");
      return;
    }
    _msgList.add(
        MediaMsgModel.init("${widget.userid} : ${msgController.text}",0, {}));

    print(
        "[Wing] sendPrivateMessage start to = ${widget.targetUid}, content = ${msgController.text}");
    var lvPushContent = new LVPushContent();
    lvPushContent.title = "push title";
    lvPushContent.body = "push content";
    lvPushContent.extra = "push extra";
    lvPushContent.click_action = "FLUTTER_NOTIFICATION_CLICK";
    RtmFlutterPlugin.sendPrivateMessage(LVIMEnumDefine.IM_SUBTYPE_TEXT,
            widget.targetUid, "msgType", msgController.text, lvPushContent)
        .then((lvimMsg) {
      print(
          '[Wing] sendPrivateMessage end result = ${lvimMsg.eCode}, dbid = ${lvimMsg.msg.dbid}, sendState = ${lvimMsg.msg.sendState} stime = ${lvimMsg.msg.stime}');
      if (lvimMsg.eCode == 0) {
        Fluttertoast.showToast(msg: "Send Msg Success");
      } else {
        Fluttertoast.showToast(msg: "Send Msg Failed code:${lvimMsg.eCode}");
      }

      setState(() {});
      msgController.text = '';
    });
  }

  // 拉黑
  void updatePrivateMessageBlack (){
    print(TAG + " 拉黑");
    RtmFlutterPlugin.updatePrivateMessageBlack('123', true)
        .then((result) {
      print(TAG + "chatroomBlackUser" + " code = $result");
    });
  }

  // 发送私信已读回执
  void sendPrivateMessageReadedAck (){
    print(TAG + " 发送私信已读回执");
    List<Map> _list = <Map>[];

    for (MediaMsgModel model in _msgList){
      if (model.json != null){
         _list.add(model.json);
      }
    }

    RtmFlutterPlugin.sendPrivateMessageReadedAck(_list, '111111')
        .then((result) {
      print(TAG + "sendPrivateMessageReadedAck" + " code = ${result}");
    });
  }

  /// 构建文件上传请求
  Future<void> sendFileMessageRequest(path, filetype) async {
    print(TAG + "构建文件上传请求");
    RtmFlutterPlugin.sendFileMessageRequest(path, filetype).then((result) {
      print(TAG + "sendFileMessageRequest" + " code = $result");
    });
  }

  ///下载文件
  void downloadFile(key, filetype, pathExtension) {
    print(TAG + "下载文件");
    RtmFlutterPlugin.downloadFile(key, filetype, pathExtension).then((result) {
      int code = result;
      print(TAG + "downloadFile" + " code = $code");
    });
  }

  ///构建音频请求
  void sendAudioMessageRequest(uid, tid, title, body, audioPath, isGroup) {
    print(TAG + "构建音频请求");
    RtmFlutterPlugin.sendAudioMessageRequest(
            uid, tid, title, body, audioPath, isGroup)
        .then((result) {
      print(TAG +
          "sendAudioMessageRequest" +
          " code = ${result.eCode} map:${result.msg.toMap()}");
      _msgList.add(MediaMsgModel.init(
          "${widget.userid} :sendAudio code = ${result.eCode}", 3,{}));
      setState(() {});
    });
  }

  ///下载音频文件
  void downloadAudio(json, index) {
    print(TAG + "下载音频文件");
    RtmFlutterPlugin.downloadAudio(json).then((result) {
      print(TAG + "downloadAudio" + " code = ${result.eCode}");
    });
  }

  ///构建图片请求
  Future<void> sendImageMessageRequest(uid, tid, title, body, isGroup) async {
    print(TAG + "构建图片请求");
    try {
      PickedFile pickedFile =
          await _picker.getImage(source: ImageSource.gallery);
      print(pickedFile.path);

      RtmFlutterPlugin.sendImageMessageRequest(
              uid, tid, title, body, pickedFile.path, isGroup)
          .then((result) {
        print(TAG + "sendImageMessageRequest" + " code = ${result.eCode}");
        _msgList.add(MediaMsgModel.init(
            "${widget.userid} :sendImage code = ${result.eCode}", 1,{}));
        setState(() {});
      });
    } catch (e) {
      print(e);
    }
  }

  ///下载图片缩略图
  void downloadThumbnailImage(json) {
    print(TAG + "下载图片缩略图");
    RtmFlutterPlugin.downloadThumbnailImage(json).then((result) {
      print(TAG + "downloadThumbnailImage" + " code = ${result.eCode}");
    });
  }

  ///下载图片文件
  void downloadImage(json,index) {
    print(TAG + "下载图片文件");
    RtmFlutterPlugin.downloadImage(json).then((result) {
      print(TAG + "downloadImage" + " code = ${result.eCode}");
      if(result.eCode == 0){
        _msgList[index].msg = result;
        setState(() { });
      }
    });
  }

  ///构建视频请求
  Future<void> sendVideoMessageRequest(uid, tid, title, body, isGroup) async {
    print(TAG + "构建视频请求");
    try {
      PickedFile pickedFile =
          await _picker.getVideo(source: ImageSource.gallery);
      print(pickedFile.path);
      RtmFlutterPlugin.sendVideoMessageRequest(
              uid, tid, title, body, pickedFile.path, isGroup)
          .then((result) {
        print(TAG + "sendVideoMessageRequest" + " code = ${result.eCode}");
        _msgList.add(MediaMsgModel.init(
            "${widget.userid} :sendVideo code = ${result.eCode}",3, {}));
        setState(() {});
      });
    } catch (e) {
      print(e);
    }
  }

  ///下载视频缩略图
  void downloadVideoThumbnailImage(json) {
    print(TAG + "下载视频缩略图");
    RtmFlutterPlugin.downloadThumbnailImage(json).then((result) {
      print(TAG + "downloadVideoThumbnailImage" + " code = ${result.eCode}");
    });
  }

  ///下载视频首帧
  void downloadVideoImage(json,index) {
    print(TAG + "下载视频首帧");
    RtmFlutterPlugin.downloadVideoImage(json).then((result) {
      print(TAG + "downloadVideoImage" + " code = ${result.eCode}");
      _msgList[index].msg = result;
      setState(() { });
    });
  }

  ///下载视频文件
  void downloadVideo(json) {
    print(TAG + "下载图片文件");
    RtmFlutterPlugin.downloadVideo(json).then((result) {
      print(TAG + "downloadVideo" + " code = ${result.eCode}");
    });
  }

  ///转换视频数据 成功返回0, 否则返回非0
  void transVideo(uid, tid, title, body, filePath, isGroup) {
    print(TAG + " 转换视频数据 成功返回0, 否则返回非0");
    RtmFlutterPlugin.transVideo(uid, tid, title, body, filePath, isGroup)
        .then((result) {
      print(TAG + "transVideo" + " code = ${result}");
    });
  }

  ///获取本地路径
  void localPathWithFileContent(key, filetype, pathExtension) {
    print(TAG + " 获取本地路径");
    RtmFlutterPlugin.localPathWithFileContent(key, filetype, pathExtension)
        .then((result) {
      print(TAG + "localPathWithFileContent" + " code = ${result}");
    });
  }

  @override
  void onEventMessageReceive(var msg) {
    print(TAG +
        " onEventMessageReceive content = ${msg.msgContent}   type = ${msg.extend1} ");
  }

  @override
  void onIMAuthFailed(
      String uid, String token, int ecode, int rcode, bool isTokenExpired) {
    print(TAG + "onIMAuthFailed ");
  }

  @override
  void onIMAuthSucceed(String uid, String token, int unReadMsgSize) {
    print(TAG + "onIMAuthSucceed ");
  }

  @override
  void onIMMessageReadedAcks(List<int> listId) {
    print(TAG + "onIMMessageReadedAcks ");
  }

  @override
  void onIMMessageReceive(var msg) {
    var msgContent = msg.msgContent.toString();
    print(TAG + " onIMMessageReceive  fromid = " + msg.fromID);
    _msgList
        .add(MediaMsgModel.init("${msg.fromID} : ${msgContent}",msg.subType, msg.toMap()));
    setState(() {});
    // 发送该消息已读
    RtmFlutterPlugin.sendMessageHasRead(msg.fromID, msg.stime);
  }

  @override
  void onIMTokenExpired(String uid, String token) {
    print(TAG + "onIMTokenExpired uid = " + uid);
  }

  @override
  void onIMUserKicked(String uid, int code) {
    print(TAG + "onIMUserKicked ");
  }

  @override
  void onQueryIMToken() {
    print(TAG + "onQueryIMToken ");
  }

  @override
  void onReadMessageReceive(var msg, int stime) {
    // stime之前的消息皆可设为已读
    print(TAG + "onReadMessageReceive : $stime");

    // _msgListTextEdit.text = msg.fromID + " : 已读时间戳为 $stime 的消息";
  }

  Widget _toolBarWidget() {
    return Container(
      padding: const EdgeInsets.all(10),
      width: MediaQuery.of(context).size.width,
      color: Colors.green,
      child: Column(
        mainAxisAlignment: MainAxisAlignment.spaceAround,
        mainAxisSize: MainAxisSize.max,
        children: [
          Row(
            children: [
              Container(
                width: MediaQuery.of(context).size.width * 0.7,
                color: Colors.white,
                height: 50,
                child: TextField(
                  autofocus: true,
                  focusNode: focusNode,
                  inputFormatters: <TextInputFormatter>[
                    LengthLimitingTextInputFormatter(500)
                  ],
                  controller: msgController,
                  onSubmitted: (value) {
                    _sendMessageAction();
                  },
                  decoration: InputDecoration(
                      isDense: true,
                      border: InputBorder.none,
                      hintStyle: TextStyle(color: Color(0xffc1c1c1)),
                      hintText: "please input content message"),
                ),
              ),
              Container(
                margin: const EdgeInsets.only(left: 20),
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () {
                    _sendMessageAction();
                  },
                  child: Text("Send"),
                ),
              ),
            ],
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    sendFileMessageRequest('path',
                        LVIMEnumDefine.IM_MEDIA_FILE_PRIVATE_FILE.toInt());
                  },
                  child: Text(
                    "文件",
                    style: TextStyle(
                        decoration: TextDecoration.lineThrough,
                        decorationColor: Colors.black),
                  ),
                ),
              ),
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    sendAudioMessageRequest(widget.userid, widget.targetUid,
                        'a', 'abc', 'audioPath', false);
                  },
                  child: Text(
                    "音频",
                    style: TextStyle(
                        decoration: TextDecoration.lineThrough,
                        decorationColor: Colors.black),
                  ),
                ),
              ),
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    sendImageMessageRequest(
                        widget.userid, widget.targetUid, 'a', 'abc', false);
                  },
                  child: Text("图片"),
                ),
              ),
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    sendVideoMessageRequest(
                        widget.userid, widget.targetUid, 'a', 'abc', false);
                  },
                  child: Text("视频"),
                ),
              ),
            ],
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceAround,
            children: [
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    updatePrivateMessageBlack();
                  },
                  child: Text(
                    "拉黑"
                  ),
                ),
              ),
              Container(
                color: Colors.red[100],
                child: TextButton(
                  onPressed: () async {
                    sendPrivateMessageReadedAck();
                  },
                  child: Text(
                    "发送私信已读回执"
                  ),
                ),
              ),
            ],
          )
        ],
      ),
    );
  }

  @override
  void onIMConnected() {
    // TODO: implement onIMConnected
    Fluttertoast.showToast(
        msg: "onIMConnected()方法被调用，网络连接",
        toastLength: Toast.LENGTH_LONG,
        gravity: ToastGravity.CENTER);
  }

  @override
  void onIMLosted() {
    // TODO: implement onIMLosted
    Fluttertoast.showToast(
        msg: "onIMLosted()方法被调用，网络断开",
        toastLength: Toast.LENGTH_LONG,
        gravity: ToastGravity.CENTER);
  }


}
