//
//  LVRTCLive.m
//  iOS_Zego
//
//  Created by Wing on 2020/8/4.
//  Copyright © 2020 Wing. All rights reserved.
//


#import "LVRTCLive.h"
#import <LinkV/LinkV.h>

#ifdef DEBUG
#define DLog(...) NSLog(__VA_ARGS__)
#else
#define DLog(...) /* */
#endif

@interface LVRTCLive ()<LVRTCEngineDelegate>

@property (nonatomic, strong) NSMutableDictionary *playDictM;

@end

@implementation LVRTCLive

// 推流参数配置
- (void)initSDK:(NSString *)appId appKey:(NSString *)appKey isTestEnv:(BOOL)isTestEnv isInternationalEnv:(BOOL)isInternationalEnv {
     
     [LVRTCEngine initSdk];
     [LVRTCEngine setUseTestEnv:isTestEnv];
    [LVRTCEngine setUseInternationalEnv:isInternationalEnv];
     [[LVRTCEngine sharedInstance] auth:appId skStr:appKey userId:@"" completion:^(LVErrorCode code) {
         if ([self.delegate respondsToSelector:@selector(authComplete:)]) {
             [self.delegate authComplete:code];
         }
     }];
}

- (void)setLogOpen:(BOOL)isOpen {
    LVLoggingSeverity log = isOpen ? kLVLoggingSeverityVerbose : kLVLoggingSeverityNone;
    [LVRTCEngine setLogLevel:log];
}

- (void)setVideoConfig:(VideoConfigLevel)configLevel {
    [[LVRTCEngine sharedInstance] setAVConfig:[[LVAVConfig alloc] initWithVideoProfile:configLevel]];
}

- (void)loginRoom:(NSString *)uid roomId:(NSString *)roomId isHost:(BOOL)isHost isAudioOnly:(BOOL)isAudioOnly; {
    self.roomId = roomId;
    self.uid = uid;
    
    [[LVRTCEngine sharedInstance] loginRoom:uid roomId:roomId isHost:isHost isOnlyAudio:isAudioOnly delegate:self];
}

- (void)logoutRoom {
    if (!self.roomId) return;
    
    [[LVRTCEngine sharedInstance] stopPublishing];
    
    NSArray *uids = self.playDictM.allKeys;
    for (NSString *uid in uids) {
        [self stopPlayingStream:uid];
    }
    
    [[LVRTCEngine sharedInstance] logoutRoom:^(LVErrorCode code) {
        NSLog(@"[Wing] logoutRoom code = %@", @(code));
    }];
}

- (void)startPreview:(UIView *)view {
    
    LVRTCDisplayView *displayView = [[LVRTCDisplayView alloc] initWithFrame:view.bounds];
    [view insertSubview:displayView atIndex:0];
    displayView.uid = self.uid;
    displayView.isLocal = true;
    displayView.viewContentMode = LVViewContentModeScaleAspectFill;
    [[LVRTCEngine sharedInstance] addDisplayView:displayView];
    [[LVRTCEngine sharedInstance] startCapture];
    
    if (!self.uid) return;
    self.playDictM[self.uid] = displayView;
}

- (void)startPublishing {
    [[LVRTCEngine sharedInstance] startPublishing];
}

- (void)startPlayingStream:(NSString *)uid inView:(UIView *)view {
    if (!uid) return;
    
    if (view) {
        LVRTCDisplayView *displayView = [[LVRTCDisplayView alloc] initWithFrame:CGRectMake(0, 0, 320, 100)];
        [view insertSubview:displayView atIndex:0];
        displayView.uid = uid;
        displayView.viewContentMode = LVViewContentModeScaleAspectFill;
        [[LVRTCEngine sharedInstance] addDisplayView:displayView];
        self.playDictM[uid] = displayView;
        
        displayView.translatesAutoresizingMaskIntoConstraints = NO;
        
        NSLayoutConstraint *leading = [NSLayoutConstraint constraintWithItem:displayView attribute:NSLayoutAttributeLeading relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeLeading multiplier:1 constant:0];
        NSLayoutConstraint *top = [NSLayoutConstraint constraintWithItem:displayView attribute:NSLayoutAttributeTop relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeTop multiplier:1 constant:0];
        NSLayoutConstraint *trailing = [NSLayoutConstraint constraintWithItem:displayView attribute:NSLayoutAttributeTrailing relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeTrailing multiplier:1 constant:0];
        NSLayoutConstraint *bottom = [NSLayoutConstraint constraintWithItem:displayView attribute:NSLayoutAttributeBottom relatedBy:NSLayoutRelationEqual toItem:view attribute:NSLayoutAttributeBottom multiplier:1 constant:0];
        
        [view addConstraints:@[leading, top, trailing, bottom]];
    }
    
    [[LVRTCEngine sharedInstance] startPlayingStream:uid];
}

- (void)stopPlayingStream:(NSString *)uid {
    if (!uid) return;
    [[LVRTCEngine sharedInstance] stopPlayingStream:uid];
    
    LVRTCDisplayView *displayView = self.playDictM[uid];
    if (displayView) {
        [[LVRTCEngine sharedInstance] removeDisplayView:displayView];
        [displayView removeFromSuperview];
    }
    
    [self.playDictM removeObjectForKey:uid];
}

/// 停止发布音视频数据
- (void)stopPublishStream {
    [[LVRTCEngine sharedInstance] stopPublishing];
}

// 转换前后置摄像头
- (void)useFrontCamera:(BOOL)isFront {
    [[LVRTCEngine sharedInstance] switchCamera:isFront ? LVRTCCameraPositionFront : LVRTCCameraPositionBack];
}
// 获取连麦SDK的名称
- (NSString *)getSolutionName { return @"LVRTCLive";}
// 获取支持连麦数
- (int)getSupportLine { return 8; }
// 获取SDK类型 3为自研，4为zego
- (SDKType)getSDKType { return SDK_TYPE_LVRTC; }
// 设置是否静音推流
- (void)enableMic:(BOOL)isMute {
    [[LVRTCEngine sharedInstance] enableMic:!isMute];
}

/// sdk版本号
- (NSString *)sdkVersion {
    return LVRTCEngine.buildVersion;
}

- (void)setPublishQualityMonitorCycle:(int)seconds {
    [LVRTCEngine setPublishQualityMonitorCycle:seconds];
}

- (void)setPlayQualityMonitorCycle:(int)seconds {
    [LVRTCEngine setPlayQualityMonitorCycle:seconds];
}

-(void)enableVideoAutoRotation:(BOOL)enable {
    [[LVRTCEngine sharedInstance] enableVideoAutoRotation:enable];
}

// 释放引擎
- (void)unInitSDK {
    [LVRTCEngine unInitSDK];
}

- (int)getMicVolume { return [[LVRTCEngine sharedInstance] getSoundLevelByUserId:self.uid]; }
/**
 * 设置美颜滤镜级别
 * @param beautyLevel 滤镜美颜级别 取值范围[0,1]，设置-1关闭
 */
- (void)setBeautyLevel:(CGFloat)beautyLevel {
    [[LVRTCEngine sharedInstance].getLVBeautyManager setBeautyLevel:beautyLevel];
}
/**
 * 设置滤镜明亮度
 * @param brightLevel 滤镜明亮度 取值范围[0,1]，设置-1关闭
 */
- (void)setBrightLevel:(CGFloat)brightLevel {
    [[LVRTCEngine sharedInstance].getLVBeautyManager setBrightLevel:brightLevel];
}

/**
 * 设置滤镜饱和度
 * @param toneLevel 滤镜饱和度 取值范围[0,1]，设置-1关闭
 */
- (void)setToneLevel:(CGFloat)toneLevel {
    [[LVRTCEngine sharedInstance].getLVBeautyManager setToneLevel:toneLevel];
}

/// 开始采集视频流
- (void)startCapture {
    [[LVRTCEngine sharedInstance] startCapture];
}

/// 停止采集视频流
- (void)stopCapture {
    [[LVRTCEngine sharedInstance] stopCapture];
}

#pragma mark - LVRTCEngineDelegate
/// 加入房间成功的回调
/// @param code 加入房间是否成功错误码
/// @param users 用户列表，如果房间中有用户存在，该列表会自动生效
- (void)OnEnterRoomComplete:(LVErrorCode)code users:(nullable NSArray<LVUser *> *)users {
    
    DLog(@"[Wing] OnEnterRoomComplete code = %@", @(code));
    if (code == LVErrorCodeSuccess) {
        if ([self.delegate respondsToSelector:@selector(onRoomConnected:)]) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.delegate onRoomConnected:self.roomId];
            });
        }
        for (LVUser *user in users) {
            if ([user.userId isEqualToString:self.uid]) continue;
            
            if ([self.delegate respondsToSelector:@selector(onAddRemoterUser:)]) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self.delegate onAddRemoterUser:user.userId];
                });
            }
        }
    } else {
        if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.delegate onRoomDisconnect:code roomId:self.roomId];
            });
        }
    }
}

- (void)OnAddRemoter:(LVUser *)user {
    DLog(@"[Wing] OnAddRemoter uid = %@", user.userId);
    if ([self.delegate respondsToSelector:@selector(onAddRemoterUser:)]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate onAddRemoterUser:user.userId];
        });
    }
}

- (void)OnDeleteRemoter:(NSString *)uid {
    DLog(@"[Wing] OnDeleteRemoter uid = %@", uid);
    
    if ([self.delegate respondsToSelector:@selector(onRemoteLeave:)]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate onRemoteLeave:uid];
        });
    }
}

- (void)OnKickOff:(NSInteger)reason roomId:(NSString *)roomId {
    DLog(@"[Wing] OnDeleteRemoter uid = %@", roomId);
    
    if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.delegate onRoomDisconnect:reason roomId:roomId];
        });
    }
}

- (void)OnRoomDisconnected:(LVErrorCode)code {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
            [self.delegate onRoomDisconnect:code roomId:self.roomId];
        }
    });
}

- (void)OnRoomReconnected {
    dispatch_async(dispatch_get_main_queue(), ^{
       if ([self.delegate respondsToSelector:@selector(onRoomConnected:)]) {
           [self.delegate onRoomConnected:self.roomId];
       }
    });
}

- (void)OnMixComplete:(LVErrorCode)code {}

- (void)OnPublishQualityUpdate:(LVVideoStatistic *)quality {
    DLog(@"[Wing] OnPublishQualityUpdate fps = %@", @(quality.videoFps));
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onVideoSizeChanged:width:height:)]) {
            [self.delegate onVideoSizeChanged:self.uid width:quality.frameWidth height:quality.frameHeight];
        }
        if ([self.delegate respondsToSelector:@selector(onPublishStreamQualityUpdate:quality:)]) {
            [self.delegate onPublishStreamQualityUpdate:self.uid quality:quality];
        }
    });
}

- (void)OnPublishStateUpdate:(LVErrorCode)code {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onPublishStateUpdate:)]) {
            [self.delegate onPublishStateUpdate:code];
        }
    });
}

- (void)OnPlayQualityUpate:(nonnull LVVideoStatistic *)quality userId:(nonnull NSString *)userId {
    DLog(@"[Wing] onPlayQualityUpdate fps = %@ uid = %@", @(quality.videoFps), userId);
    dispatch_async(dispatch_get_main_queue(), ^{

        if ([self.delegate respondsToSelector:@selector(onVideoSizeChanged:width:height:)] && quality.frameWidth > 0) {
            [self.delegate onVideoSizeChanged:userId width:quality.frameWidth height:quality.frameHeight];
        }
        if ([self.delegate respondsToSelector:@selector(onRemoteQualityUpdate:quality:)]) {
            [self.delegate onRemoteQualityUpdate:userId quality:quality];
        }
    });
}

- (void)OnPlayStateUpdate:(LVErrorCode)code userId:(nonnull NSString *)userId {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onPlayStateUpdate:userId:)]) {
            [self.delegate onPlayStateUpdate:code userId:userId];
        }
    });
}

- (void)OnExitRoomComplete {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
            [self.delegate onRoomDisconnect:0 roomId:self.roomId];
        }
        
        self.uid = nil;
        self.roomId = nil;
    });
}

- (void)OnRoomDisconnect:(LVErrorCode)code {
    dispatch_async(dispatch_get_main_queue(), ^{
        if ([self.delegate respondsToSelector:@selector(onRoomDisconnect:roomId:)]) {
            [self.delegate onRoomDisconnect:0 roomId:self.roomId];
        }
    });
}

- (void)OnAudioData:(NSString *)uid audio_data:(const void *)audio_data bits_per_sample:(int)bits_per_sample sample_rate:(int)sample_rate number_of_channels:(size_t)number_of_channels number_of_frames:(size_t)number_of_frames {
    
}

- (void)OnAudioMixStream:(nonnull const int16_t *)data samples:(int)samples nchannel:(int)nchannel flag:(LVAudioRecordType)flag {
    
}


- (void)OnAudioVolumeUpdate:(nonnull NSArray<LVAudioVolume *> *)soundLevels {
    
}


- (void)OnCaptureSoundLevelUpdate:(nonnull LVAudioVolume *)captureSoundLevel {
    
}

#pragma mark - 懒加载
- (NSMutableDictionary *)playDictM {
    if (!_playDictM) {
        _playDictM = [NSMutableDictionary dictionary];
    }
    return _playDictM;
}

@end
