#import "RtmFlutterPlugin.h"
#if __has_include(<rtm_flutter_plugin/rtm_flutter_plugin-Swift.h>)
#import <rtm_flutter_plugin/rtm_flutter_plugin-Swift.h>
#else
// Support project import fallback if the generated compatibility header
// is not copied when this plugin is created as a library.
// https://forums.swift.org/t/swift-static-libraries-dont-copy-generated-objective-c-header/19816
//#import "rtm_flutter_plugin-Swift.h"
#endif

#import "LVRTMEngine.h"
#import "FlutterSurfaceViewFactory.h"

@interface RtmFlutterPlugin ()<FlutterPlugin, RoomCallbackProtocl, LVIMModuleEventDelegate, LVIMReceiveMessageDelegate>

@property (nonatomic, strong) FlutterMethodChannel *channel;
@property (nonatomic, strong) LVRTMEngine *engine;

@property (nonatomic, strong) FlutterResult initResult;
@property (nonatomic, copy) NSString *uid;
@property (nonatomic, assign) BOOL isRequestIMToken;
@property (nonatomic, assign) bool imStatus;

@end

@implementation RtmFlutterPlugin
+ (void)registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    
    FlutterMethodChannel *channel = [FlutterMethodChannel methodChannelWithName:@"rtm_flutter_plugin" binaryMessenger:registrar.messenger];
    RtmFlutterPlugin *plugin = [RtmFlutterPlugin new];
    plugin.channel = channel;
    [registrar addMethodCallDelegate:plugin channel:channel];
    
    // 注册我们创建的view，这里的withId需要和flutter侧相同
    [registrar registerViewFactory:[FlutterSurfaceViewFactory new] withId:@"surface"];
}

- (void)handleMethodCall:(FlutterMethodCall *)call result:(FlutterResult)result {
    if ([call.method isEqualToString:@"getPlatformVersion"]) {
        result([UIDevice currentDevice].systemVersion);
    } else if ([call.method isEqualToString:@"initRtcAndImSDK"]) {
        [self initRtcAndImSDK:call result:result];
    } else if ([call.method isEqualToString:@"initRtcSDK"]) {
        [self initRtcSDK:call result:result];
    } else if ([call.method isEqualToString:@"initImSDK"]) {
        [self initImSDK:call result:result];
    }  else if ([call.method isEqualToString:@"loginUser"]) {
        [self loginUser:call result:result];
    } else if ([call.method isEqualToString:@"setLogOpen"]) {
        [self setLogOpen:call result:result];
    }  else if ([call.method isEqualToString:@"setVideoConfig"]) {
        [self setVideoConfig:call result:result];
    }  else if ([call.method isEqualToString:@"getSdkVersion"]) {
        [self getSdkVersion:call result:result];
    }  else if ([call.method isEqualToString:@"sendPrivateMessage"]) {
        [self sendPrivateMessage:call result:result];
    } else if ([call.method isEqualToString:@"sendRoomMessage"]) {
        [self sendRoomMessage:call result:result];
    }  else if ([call.method isEqualToString:@"unInitRtcAndImSDK"]) {
        [self unInitRtcAndImSDK:call result:result];
    }  else if ([call.method isEqualToString:@"unInitRtcSDK"]) {
        [self unInitRtcSDK:call result:result];
    }  else if ([call.method isEqualToString:@"unInitImSDK"]) {
        [self unInitImSDK:call result:result];
    }  else if ([call.method isEqualToString:@"loginRoom"]) {
        [self loginRoom:call result:result];
    }  else if ([call.method isEqualToString:@"setPublishQualityMonitorCycle"]) {
        [self setPublishQualityMonitorCycle:call result:result];
    }  else if ([call.method isEqualToString:@"startPublishing"]) {
        [self startPublishing:call result:result];
    }  else if ([call.method isEqualToString:@"setPlayQualityMonitor"]) {
        [self setPlayQualityMonitor:call result:result];
    }  else if ([call.method isEqualToString:@"startPlayingStream"]) {
        [self startPlayingStream:call result:result];
    } else if ([call.method isEqualToString:@"stopPlayingStream"]) {
        [self stopPlayingStream:call result:result];
    }  else if ([call.method isEqualToString:@"logoutRoom"]){
        [self logoutRoom:call result:result];
    }  else if ([call.method isEqualToString:@"logoutUser"]){
             [self logoutUser:call result:result];
    }  else if ([call.method isEqualToString:@"chatroomBlackUser"]) {
        [self chatroomBlackUser:call result:result];
    } else if ([call.method isEqualToString:@"chatroomGagUser"]) {
        [self chatroomGagUser:call result:result];
    } else if ([call.method isEqualToString:@"chatroomData"]) {
        [self chatroomData:call result:result];
    } else if ([call.method isEqualToString:@"chatroomUserData"]) {
        [self chatroomUserData:call result:result];
    } else if ([call.method isEqualToString:@"stopPreview"]) {
        [self stopPreview:call result:result];
    }  else if ([call.method isEqualToString:@"stopPublishStream"]) {
        [self stopPublishStream:call result:result];
    }  else if ([call.method isEqualToString:@"switchCamera"]) {
        [self switchCamera:call result:result];
    }  else if ([call.method isEqualToString:@"setMuteOutput"]) {
        [self setMuteOutput:call result:result];
    }  else if ([call.method isEqualToString:@"isIMAuthed"]) {
        [self isIMAuthed:call result:result];
    } else if ([call.method isEqualToString:@"isIMLoginSucceed"]) {
        [self isIMLoginSucceed:call result:result];
    } else if ([call.method isEqualToString:@"setBeautyLevel"]) {
        [self setBeautyLevel:call result:result];
    } else if ([call.method isEqualToString:@"setIMToken"]) {
        [self setIMToken:call result:result];
    } else if ([call.method isEqualToString:@"requestDebugToken"]) {
        [self requestDebugToken:call result:result];
    } else if ([call.method isEqualToString:@"startCapture"]) {
        [self startCapture];
    } else if ([call.method isEqualToString:@"stopCapture"]) {
        [self stopCapture];
    } else if ([call.method isEqualToString:@"sendEventMessage"]) {
        [self sendEventMessage:call result:result];
    } else if ([call.method isEqualToString:@"deleteLocalPrivateSession"]) {
        [self deleteLocalPrivateSession:call result:result];
    } else if ([call.method isEqualToString:@"clearSessionUnreadMsg"]) {
        [self clearSessionUnreadMsg:call result:result];
    } else if ([call.method isEqualToString:@"clearAllUnreadMsg"]) {
        [self clearAllUnreadMsg:call result:result];
    } else if ([call.method isEqualToString:@"updateMsgAsReadByStime"]) {
        [self updateMsgAsReadByStime:call result:result];
    } else if ([call.method isEqualToString:@"queryLocalPrivateHistoryMessage"]) {
        [self queryLocalPrivateHistoryMessage:call result:result];
    } else if ([call.method isEqualToString:@"querySessionList"]) {
        [self querySessionList:call result:result];
    } else if ([call.method isEqualToString:@"sendMessage"]) {
        [self sendMessage:call result:result];
    } else if ([call.method isEqualToString:@"queryAllSessionList"]) {
        [self queryAllSessionList:call result:result];
    } else if ([call.method isEqualToString:@"setPrivateDBStorageMax"]) {
        [self setPrivateDBStorageMax:call result:result];
    } else if ([call.method isEqualToString:@"queryUnReadMessageNum"]) {
        [self queryUnReadMessageNum:call result:result];
    } else if ([call.method isEqualToString:@"uploadPushToken"]) {
        [self uploadPushToken:call result:result];
    } else if([call.method isEqualToString:@"updatePrivateMessageBlack"]){
        [self updatePrivateMessageBlack:call result:result];
    } else if([call.method isEqualToString:@"getPrivateMessageBlack"]){
        [self getPrivateMessageBlack:call result:result];
    } else if ([call.method isEqualToString:@"sendPrivateMessageReadedAck"]){
        [self sendPrivateMessageReadedAck:call result:result];
    } else if ([call.method isEqualToString:@"sendFileMessageRequest"]){
        [self sendFileMessageRequest:call result:result];
    } else if ([call.method isEqualToString:@"downloadFile"]){
        [self downloadFile:call result:result];
    } else if ([call.method isEqualToString:@"sendAudioMessageRequest"]){
        [self sendAudioMessageRequest:call result:result];
    } else if ([call.method isEqualToString:@"downloadAudio"]){
        [self downloadAudio:call result:result];
    } else if ([call.method isEqualToString:@"sendImageMessageRequest"]){
        [self sendImageMessageRequest:call result:result];
    } else if ([call.method isEqualToString:@"downloadThumbnailImage"]){
        [self downloadThumbnailImage:call result:result];
    } else if ([call.method isEqualToString:@"downloadImage"]){
        [self downloadImage:call result:result];
    } else if ([call.method isEqualToString:@"sendVideoMessageRequest"]){
        [self sendVideoMessageRequest:call result:result];
    } else if ([call.method isEqualToString:@"transVideo"]){
        [self transVideo:call result:result];
    } else if ([call.method isEqualToString:@"downloadVideoThumbnailImage"]){
        [self downloadVideoThumbnailImage:call result:result];
    } else if ([call.method isEqualToString:@"downloadVideoImage"]){
        [self downloadVideoImage:call result:result];
    } else if ([call.method isEqualToString:@"downloadVideo"]){
        [self downloadVideo:call result:result];
    } else if ([call.method isEqualToString:@"localPathWithFileContent"]){
        [self localPathWithFileContent:call result:result];
    } else if ([call.method isEqualToString:@"queryRemoteSessionMessage"]){
        [self queryRemoteSessionMessage:call result:result];
    }else{
        result(FlutterMethodNotImplemented);
    }
}

#pragma mark - Flutter Call Native

/// 拉取远程某个会话的历史消息
-(void)queryRemoteSessionMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *tuid = dict[@"tuid"];
    int sequence = [dict[@"sequence"] unsignedIntValue];
    int size = [dict[@"size"] unsignedIntValue];
    int type = [dict[@"type"] unsignedIntValue];
    
    [[LVIMSDK sharedInstance].msgManager queryRemoteSessionMessage:tuid sequence:sequence size:size type:type callback:^(int32_t ecode, int32_t rcode, int32_t rstatus, NSArray *messages) {
        NSMutableArray *array = [NSMutableArray array];
        for (id msg in messages) {
            [array addObject:[self msgToDict:msg]];
        }
        result(@{@"ecode":@(ecode),@"rcode":@(rcode),@"messages":array ?: @[]});
    }];
}

/// 发送私信已读回执
-(void)sendPrivateMessageReadedAck:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSArray *listData = dict[@"listMsg"];
    NSString *sessionId = dict[@"sessionId"];
    NSMutableArray *listMsg = [NSMutableArray array];
    for (NSDictionary *data in listData) {
        if (data.count != 0) {
            NSDictionary *newData = [self removeNull:data];
            [listMsg addObject:[LVIMMessage fromFlutterJsonDictionary:newData]];
        }
    }

    [[LVIMSDK sharedInstance].msgManager sendPrivateMessageReadedAckWithMessageList:listMsg sessionid:sessionId context:nil callBack:^(int ecode, int rcode) {
        result(@(ecode));
    }];
}
 
/// 构建文件上传请求
-(void)sendFileMessageRequest:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *path = dict[@"path"];
    int filetype = [dict[@"filetype"] unsignedIntValue];

//    NSString *testPath = [[NSBundle mainBundle] pathForResource:@"a" ofType:@"txt"];
//    path = testPath;
    
    LVIMFileContent *content = [[LVIMFileContent alloc] initWithPath:path fileType:filetype];
    [content uploadFileWithProgress:^(float progress) {
    } complete:^(NSError *error) {
        if (!error) {
            result(0);
        }else{
            result(@(error.code));
        }
    }];
}

/// 下载文件
-(void)downloadFile:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *key = dict[@"key"];
    NSString *pathExtension = dict[@"pathExtension"];
    int filetype = [dict[@"filetype"] unsignedIntValue];
    LVIMFileContent *content = [[LVIMFileContent alloc] initWithKey:key fileType:filetype pathExtension:pathExtension];
    [content downloadFileWithProgress:^(float progress) {
    } complete:^(NSError *error) {
        if (!error) {
            result(0);
        }else{
            result(@(error.code));
        }
    }];

}

/// 构建音频请求
-(void)sendAudioMessageRequest:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *uid = dict[@"uid"];
    NSString *tid = dict[@"tid"];
    NSString *title = dict[@"title"];
    NSString *body = dict[@"body"];
    NSString *audioPath = dict[@"audioPath"];
    BOOL isGroup = [dict[@"isGroup"] boolValue];

    LVIMAudioMessage *audioMessage = [[LVIMAudioMessage alloc] init];
    audioMessage.needSaveToDB = YES;
    
    LVPushContent *pushContent = [[LVPushContent alloc] init];
    pushContent.title = title;
    pushContent.body = body;
    
    if (isGroup) {
        [audioMessage buildAudioGroupRequest:uid tid:tid type:@"AUDIO" audioPath:audioPath content:@"AUDIO" pushContent:pushContent];
    }else{
        [audioMessage buildAudioPrivateRequest:uid tid:tid type:@"AUDIO" audioPath:audioPath content:@"AUDIO" pushContent:pushContent];
    }
    [audioMessage uploadFilesWithProgress:^(float progress) {
    } complete:^(NSError *error) {
        [self _sendMediaMessages:audioMessage error:error result:result];
    }];
    
    
}

/// 下载音频文件
-(void)downloadAudio:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];

    LVIMAudioMessage *audioMessage = (LVIMAudioMessage *)[LVIMAudioMessage fromFlutterJsonDictionary:dict];
    if (audioMessage.mSubType == LVIM_MESSAGE_SUBTYPE_AUDIO) {
        if (!audioMessage.audioFile.localPath) {
            [audioMessage downloadAudioWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:audioMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:audioMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:audioMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
}

/// 构建图片请求
-(void)sendImageMessageRequest:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *uid = dict[@"uid"];
    NSString *tid = dict[@"tid"];
    NSString *title = dict[@"title"];
    NSString *body = dict[@"body"];
    NSString *imagePath = dict[@"imagePath"];
    BOOL isGroup = [dict[@"isGroup"] boolValue];

    LVIMImageMessage *imageMessage = [[LVIMImageMessage alloc] init];
    imageMessage.needSaveToDB = YES;
    LVPushContent *pushContent = [[LVPushContent alloc] init];
    pushContent.title = title;
    pushContent.body = body;
    UIImage *image = [UIImage imageWithContentsOfFile:imagePath];
    if (isGroup) {
        [imageMessage buildImageGroupRequest:uid tid:tid type:@"IMAGE" image:image originImage:NO content:@"IMAGE" imageType:(LVIM_IMAGE_FILE_TYPE_JPEG) pushContent:pushContent];
    }else{
        [imageMessage buildImagePrivateRequest:uid tid:tid type:@"IMAGE" image:image originImage:NO content:@"IMAGE" imageType:(LVIM_IMAGE_FILE_TYPE_JPEG) pushContent:pushContent];
    }
 
    [imageMessage uploadFilesWithProgress:^(float progress) {
    } complete:^(NSError *error) {
        [self _sendMediaMessages:imageMessage error:error result:result];
    }];

}

/// 下载缩略图
-(void)downloadThumbnailImage:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
 
    LVIMImageMessage *imageMessage = (LVIMImageMessage *)[LVIMImageMessage fromFlutterJsonDictionary:dict];
    if (imageMessage.mSubType == LVIM_MESSAGE_SUBTYPE_IMAGE) {
        if (!imageMessage.imageFile.localPath) {
            [imageMessage downloadThumbnailImageWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:imageMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:imageMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:imageMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
}

/// 下载图片
-(void)downloadImage:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
 
    LVIMImageMessage *imageMessage = (LVIMImageMessage *)[LVIMImageMessage fromFlutterJsonDictionary:dict];
    if (imageMessage.mSubType == LVIM_MESSAGE_SUBTYPE_IMAGE) {
        if (!imageMessage.imageFile.localPath) {
            [imageMessage downloadImageWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:imageMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:imageMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:imageMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
    
}
    
/// 构建视频请求
-(void)sendVideoMessageRequest:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *uid = dict[@"uid"];
    NSString *tid = dict[@"tid"];
    NSString *title = dict[@"title"];
    NSString *body = dict[@"body"];
    NSString *filePath = dict[@"filePath"];
    BOOL isGroup = [dict[@"isGroup"] boolValue];

    LVIMVideoMessage *videoMessage = [[LVIMVideoMessage alloc] init];
    videoMessage.needSaveToDB = YES;
    LVPushContent *pushContent = [[LVPushContent alloc] init];
    pushContent.title = title;
    pushContent.body = body;
    if (isGroup) {
        [videoMessage buildVideoGroupRequest:uid tid:tid type:@"VIDEO" videoPath:filePath content:@"VIDEO" pushContent:pushContent];
    }else{
        [videoMessage buildVideoPrivateRequest:uid tid:tid type:@"VIDEO" videoPath:filePath content:@"VIDEO" pushContent:pushContent];
    }

    [videoMessage transVideoWithCompletionHandler:^(BOOL rs) {
        if (rs) {
            [videoMessage uploadFilesWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                [self _sendMediaMessages:videoMessage error:error result:result];
            }];
        }else{
            result(@(-1));
        }
    }];
    
}

/// 转换视频数据 成功返回0, 否则返回非0
-(void)transVideo:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *uid = dict[@"uid"];
    NSString *tid = dict[@"tid"];
    NSString *title = dict[@"title"];
    NSString *body = dict[@"body"];
    NSString *filePath = dict[@"filePath"];
    BOOL isGroup = [dict[@"isGroup"] boolValue];
    
    LVIMVideoMessage *videoMessage = [[LVIMVideoMessage alloc] init];
    videoMessage.needSaveToDB = YES;
    LVPushContent *pushContent = [[LVPushContent alloc] init];
    pushContent.title = title;
    pushContent.body = body;
    if (isGroup) {
        [videoMessage buildVideoGroupRequest:uid tid:tid type:@"VIDEO" videoPath:filePath content:@"VIDEO" pushContent:pushContent];
    }else{
        [videoMessage buildVideoPrivateRequest:uid tid:tid type:@"VIDEO" videoPath:filePath content:@"VIDEO" pushContent:pushContent];
    }

    [videoMessage transVideoWithCompletionHandler:^(BOOL rs) {
        if (rs) {
            result(@(0));
        }else{
            result(@(-1));
        }
    }];
    
}


/// 下载视频缩略图
-(void)downloadVideoThumbnailImage:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
 
    LVIMVideoMessage *videoMessage = (LVIMVideoMessage *)[LVIMVideoMessage fromFlutterJsonDictionary:dict];
    if (videoMessage.mSubType == LVIM_MESSAGE_SUBTYPE_VIDEO) {
        if (!videoMessage.imageFile.localPath) {
            [videoMessage downloadThumbnailImageWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:videoMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
    
}

/// 下载视频首帧图片
-(void)downloadVideoImage:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
 
    LVIMVideoMessage *videoMessage = (LVIMVideoMessage *)[LVIMVideoMessage fromFlutterJsonDictionary:dict];
    if (videoMessage.mSubType == LVIM_MESSAGE_SUBTYPE_VIDEO) {
        if (!videoMessage.imageFile.localPath) {
            [videoMessage downloadImageWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:videoMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
    
}

/// 下载视频
-(void)downloadVideo:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
 
    LVIMVideoMessage *videoMessage = (LVIMVideoMessage *)[LVIMVideoMessage fromFlutterJsonDictionary:dict];
    if (videoMessage.mSubType == LVIM_MESSAGE_SUBTYPE_VIDEO) {
        if (!videoMessage.imageFile.localPath) {
            [videoMessage downloadVideoWithProgress:^(float progress) {
            } complete:^(NSError *error) {
                if (!error) {
                    //正常展示
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(0);
                    result(dictM);
                }else{
                    NSLog(@"下载失败");
                    NSMutableDictionary *dictM = [self msgToDict:videoMessage];
                    dictM[@"eCode"] = @(-1);
                    result(dictM);
                }
            }];
        }else{
            NSMutableDictionary *dictM = [self msgToDict:videoMessage];
            dictM[@"eCode"] = @(0);
            result(dictM);
        }
    }
    
}

/// 根据文件的key获取文件本地路径
-(void)localPathWithFileContent:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *key = dict[@"key"];
    NSString *pathExtension = dict[@"pathExtension"];
    int filetype = [dict[@"filetype"] unsignedIntValue];
    
    LVIMFileContent *content = [[LVIMFileContent alloc] initWithKey:key fileType:filetype pathExtension:pathExtension];
    NSString *localPath = [LVIMFileUtils localPathWithFileContent:content];
    result(localPath);

}



/**
 * @brief 直播间用户拉黑
 *
 * rid      房间ID
 * blackid  拉黑用户ID
 * isAdd    添加或者删除
 */
-(void)chatroomBlackUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *rid = dict[@"rid"];
    NSString *blackid = dict[@"blackid"];
    BOOL isAdd = [dict[@"isAdd"] boolValue];
    
    int send_rs = [[LVIMSDK sharedInstance].chatroomManager chatroomBlack:rid blackUser:blackid isAdd:isAdd context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        result(@(ecode));
    }];
    
    if (send_rs != 0) {
        result(@(send_rs));
    }
}

/**
 * @brief 直播间用户禁言
 *
 * rid      房间ID
 * gagid  禁言用户ID
 * time   禁言时间
 * isAdd    添加或者删除
 */
-(void)chatroomGagUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *rid = dict[@"rid"];
    NSString *gagid = dict[@"gagid"];
    int time = [dict[@"time"] intValue];
    BOOL isAdd = [dict[@"isAdd"] boolValue];
    
    int send_rs = [[LVIMSDK sharedInstance].chatroomManager chatroomGag:rid gagUser:gagid time:time isAdd:isAdd context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        result(@(ecode));
    }];
    
    if (send_rs != 0) {
        result(@(send_rs));
    }
}

/**
 *
 * @brief 房间数据
 * rid 房间ID
 * opType 操作方式/增删改查
 * key key
 * value value
 */
-(void)chatroomData:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *rid = dict[@"rid"];
    int opType = [dict[@"opType"] intValue];
    NSString *key = dict[@"key"];
    NSString *value = dict[@"value"];
    
    int send_rs = [[LVIMSDK sharedInstance].chatroomManager chatroomData:rid opType:opType key:key value:value context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSDictionary *dicationry) {
        NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }

}

/**
 * @brief 房间用户数据
 * rid 房间ID
 * opType 操作方式/增删改查
 * key key
 * value value
 */
-(void)chatroomUserData:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *rid = dict[@"rid"];
    int opType = [dict[@"opType"] intValue];
    NSString *key = dict[@"key"];
    NSString *value = dict[@"value"];
    
    int send_rs = [[LVIMSDK sharedInstance].chatroomManager chatroomUserData:rid opType:opType key:key value:value context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSDictionary *dicationry) {
        NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }
    
}

/**
 *
 * 用户私信黑名单管理
 * uid 被拉黑的用户ID
 * isAdd true为添加,false为删除
 * 成功返回0, 否则返回非0
 */
-(void)updatePrivateMessageBlack:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *uid = dict[@"uid"];
    BOOL isAdd = [dict[@"isAdd"] boolValue];
    int send_rs = [[LVIMSDK sharedInstance] updatePrivateMessageBlackWithUid:uid isAdd:isAdd context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        result(@(ecode));
    }];
    
    if (send_rs != 0) {
        result(@(send_rs));
    }
    
}

/**
 * 获取用户私信黑名单列表
 * index 分页下标,第一页传入0，例如,每页20条数据,第一页传0,第二页传20
 * 成功返回0, 否则返回非0
 **/
-(void)getPrivateMessageBlack:(FlutterMethodCall *)call result:(FlutterResult)result {
    NSDictionary *dict = [self removeNull:call.arguments];
    int index = [dict[@"index"] intValue];
    
    int send_rs = [[LVIMSDK sharedInstance] getPrivateMessageBlackWithIndex:index context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSArray *list) {
        result(@(ecode));
    }];
    if (send_rs != 0) {
        result(@(send_rs));
    }
    
}

- (void)initRtcAndImSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    self.initResult = result;
    
    NSString *rtcAppID = dict[@"rtcAppID"];
    NSString *rtcAppKey = dict[@"rtcAppKey"];
    NSString *imAppID = dict[@"imAppID"];
    NSString *imAppKey = dict[@"imAppKey"];
    int sdkType = [dict[@"sdkType"] intValue];
    BOOL isDebugEnv = [dict[@"isDebugEnv"] boolValue];
    BOOL isInternationalEnv = [dict[@"isInternationalEnv"] boolValue];
    
    __weak typeof(self) weakSelf = self;
    self.engine = [LVRTMEngine createEngine:sdkType rtcAppId:rtcAppID rtcAppKey:rtcAppKey imAppId:imAppID imAppKey:imAppKey isTestEnv:isDebugEnv isInternationalEnv:isInternationalEnv delegate:self imStatusHandler:^(BOOL status) {
        __strong typeof(self) strongSelf = weakSelf;
        strongSelf.imStatus = status;
    }];
}

- (void)initRtcSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    self.initResult = result;

    NSString *rtcAppID = dict[@"rtcAppID"];
    NSString *rtcAppKey = dict[@"rtcAppKey"];
    int sdkType = [dict[@"sdkType"] intValue];
    BOOL isDebugEnv = [dict[@"isDebugEnv"] boolValue];
    BOOL isInternationalEnv = [dict[@"isInternationalEnv"] boolValue];

    self.engine = [LVRTMEngine createEngine:sdkType rtcAppId:rtcAppID rtcAppKey:rtcAppKey isTestEnv:isDebugEnv isInternationalEnv:isInternationalEnv delegate:self];
}

- (void)initImSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
 
    NSString *imAppID = dict[@"imAppID"];
    NSString *imAppKey = dict[@"imAppKey"];
    BOOL isDebugEnv = [dict[@"isDebugEnv"] boolValue];
 
    self.engine = [LVRTMEngine createEngine:SDK_TYPE_LVRTC imAppId:imAppID imAppKey:imAppKey isTestEnv:isDebugEnv delegate:self imStatusHandler:^(BOOL imStatus) {
        result(@(imStatus?1:0));
    }];
}

- (void)unInitRtcAndImSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine unInitRtcAndImSDK];
    result(@(YES));
}

- (void)unInitRtcSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine unInitRtcSDK];
    result(@(YES));
}

- (void)unInitImSDK:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine unInitImSDK];
    result(@(YES));
}

- (void)loginUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    self.uid = dict[@"userId"];
    int loginType = [dict[@"loginType"] intValue];
    int rs = [self.engine loginIM:dict[@"userId"] loginType:loginType];
    result(@(rs));
    
    NSLog(@"[Wing] loginIM uid = %@", self.uid);
}

- (void)setLogOpen:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setLogOpen:[dict[@"isOpen"] boolValue]];
    result(@(YES));
}

- (void)setVideoConfig:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setVideoConfig:[dict[@"videoConfigLevel"] integerValue]];
    result(@(YES));
}

- (void)getSdkVersion:(FlutterMethodCall *)call result:(FlutterResult)result {
    result([self.engine sdkVersion]);
}

- (void)sendPrivateMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int sub = [dict[@"sub"] intValue];
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    NSString *pushTitle = dict[@"pushTitle"];
    NSString *pushBody = dict[@"pushBody"];
    NSString *pushExtra = dict[@"pushExtra"];
    NSString *click_action = dict[@"pushClickAction"];
    
    LVPushContent *push;
    if (pushTitle || pushBody || pushExtra || click_action) {
        push = [LVPushContent new];
        push.title = pushTitle;
        push.body = pushBody;
        push.extra = pushExtra;
        push.click_action = click_action;
    }
    
    LVIMMessage *msg = [LVIMMessage new];
    int rs = [msg buildPrivateRequest:sub fid:self.uid tid:tid type:type content:content pushContent:push extend3:nil extend4:nil targetAppID:nil targetAppUID:nil];
    // 构建消息失败
    if (rs != 0) {
        result(@{@"eCode": @(rs)});
        return;
    }
    
    // 构建消息成功
    int send_rs = [[LVIMSDK sharedInstance] sendMessage:msg context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        dictM[@"rcode"] = @(rcode);
        result(dictM);
    }];
    
    // 发送消息失败
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }
}

- (void)sendRoomMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine sendRoomMessage:dict[@"targetId"] content:dict[@"msgContent"] msgType:dict[@"msgType"] complete:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        result(dictM);
    }];
}


- (void)loginRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine loginRoom:dict[@"userId"] roomId:dict[@"roomId"] isHost:[dict[@"isHost"] boolValue] isAudioOnly:[dict[@"isAudioOnly"] boolValue]];
    result(@(YES));
}

- (void)setPublishQualityMonitorCycle:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setPublishQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)startPublishing:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine startPublishing];
    result(@(YES));
}

- (void)setPlayQualityMonitor:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine setPlayQualityMonitorCycle:[dict[@"seconds"] intValue]];
    result(@(YES));
}

- (void)startPlayingStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine startPlayingStream:dict[@"streamId"] inView:nil];
    result(@(YES));
}

- (void)stopPlayingStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine stopPlayingStream:dict[@"streamId"]];
    result(@(YES));
}

- (void)logoutRoom:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine logoutRoom];
    result(@(YES));
}


- (void)logoutUser:(FlutterMethodCall *)call result:(FlutterResult)result {
    [[LVIMSDK sharedInstance] logout];
    result(@(YES));
}


- (void)stopPreview:(FlutterMethodCall *)call result:(FlutterResult)result {
    
}

- (void)stopPublishStream:(FlutterMethodCall *)call result:(FlutterResult)result {
    [self.engine stopPublishStream];
    result(@(YES));
}

- (void)switchCamera:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine useFrontCamera:[dict[@"isFrontCamera"] boolValue]];
    result(@(YES));
}

- (void)setMuteOutput:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [self.engine enableMic:[dict[@"isMute"] boolValue]];
    result(@(YES));
}

- (void)isIMAuthed:(FlutterMethodCall *)call result:(FlutterResult)result {
    result(@([[LVIMSDK sharedInstance] isAuthed]));
}

- (void)isIMLoginSucceed:(FlutterMethodCall *)call result:(FlutterResult)result {
    result(@([[LVIMSDK sharedInstance] isAppUserLoginSucceed]));
}

- (void)setBeautyLevel:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int beauty = [dict[@"beauty"] intValue];
    float level = [dict[@"level"] floatValue];
    
    switch (beauty) {
        case 0:
            [self.engine setBeautyLevel:level];
            break;
        case 1:
            [self.engine setBrightLevel:level];
            break;
        case 2:
            [self.engine setToneLevel:level];
            break;
        default:
            break;
    }
    
    result(@(YES));
}

- (void)setIMToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    [[LVIMSDK sharedInstance] setIMToken:dict[@"userId"] token:dict[@"imToken"]];
    result(@(YES));
}

- (void)requestDebugToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    [[LVIMSDK sharedInstance] requestDebugToken:^(NSInteger errorCode, NSDictionary *responseDict) {
        NSString *token = @"";
        if ([responseDict isKindOfClass:[NSDictionary class]]) {
            token = [NSString stringWithFormat:@"%@", responseDict[@"data"]];
        }
        result(@{@"code": @(errorCode), @"result": token});
    }];
}

- (void)startCapture {
    [self.engine startCapture];
}

- (void)stopCapture {
    [self.engine stopCapture];
}

- (void)sendEventMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    NSString *pushTitle = dict[@"pushTitle"];
    NSString *pushBody = dict[@"pushBody"];
    NSString *pushExtra = dict[@"pushExtra"];
    NSString *click_action = dict[@"pushClickAction"];
    
    LVPushContent *push;
    if (pushTitle || pushBody || pushExtra || click_action) {
        push = [LVPushContent new];
        push.title = pushTitle;
        push.body = pushBody;
        push.extra = pushExtra;
        push.click_action = click_action;
    }
    
    LVIMMessage *msg = [LVIMMessage new];
    int rs = [msg buildEventRequest:self.uid tid:tid type:type content:content pushContent:push targetAppID:nil targetAppUID:nil];
    // 构建消息失败
    if (rs != 0) {
        result(@{@"eCode": @(rs)});
        return;
    }
    
    // 构建消息成功
    int send_rs = [[LVIMSDK sharedInstance] sendMessage:msg context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        dictM[@"rcode"] = @(rcode);
        result(dictM);
    }];
    
    // 发送消息失败
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }
}

- (void)deleteLocalPrivateSession:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    int num = [[LVIMSDK sharedInstance].msgManager deleteLocalPrivateHistoryMessage:uid];
    result(@(num));
}

- (void)clearSessionUnreadMsg:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    int type = [dict[@"type"] intValue];
    int num = [[LVIMSDK sharedInstance].conversationManager clearSessionUnreadMsg:uid type:type];
    result(@(num));
}

- (void)clearAllUnreadMsg:(FlutterMethodCall *)call result:(FlutterResult)result {
    int num = [[LVIMSDK sharedInstance].conversationManager clearAllUnreadMsg:(LVIMDB_TYPE_PRIVATE)];
    result(@(num));
}

- (void)updateMsgAsReadByStime:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    NSInteger stime = [dict[@"stime"] integerValue];
    int num = [[LVIMSDK sharedInstance].conversationManager updateMsgAsReadByStime:uid stime:stime type:(LVIMDB_TYPE_PRIVATE)];
    result(@(num));
}

- (void)queryLocalPrivateHistoryMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    NSString *uid = dict[@"uid"];
    NSInteger dbid = [dict[@"dbid"] integerValue];
    int limit = [dict[@"limit"] intValue];
    dbid = dbid == 0 ? NSIntegerMax : dbid;
    
    NSMutableArray<LVIMMessage *> *arrM = [NSMutableArray arrayWithCapacity:limit];
    int num = [[LVIMSDK sharedInstance].msgManager queryLocalPrivateHistoryMessage:uid dbid:dbid limit:limit desc:YES rmsgs:arrM];
    
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"result"] = @(num);
    if (num <= 0) {
        result(dictM);
        return;
    }
    
    NSMutableArray *msgs = [NSMutableArray arrayWithCapacity:num];
    for (LVIMMessage *msg in arrM) {
        [msgs addObject:[self msgToDict:msg]];
    }
    dictM[@"msgs"] = msgs;
    result(dictM);
}

- (void)querySessionList:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    int type = [dict[@"type"] intValue];
    NSMutableArray<LVIMSession *> *arrM = [NSMutableArray array];
    int num = [[LVIMSDK sharedInstance].conversationManager querySessionList:arrM type:type];
    
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"result"] = @(num);
    
    if (num <= 0) {
        result(dictM);
        return;
    }
    
    NSMutableArray *msgs = [NSMutableArray arrayWithCapacity:num];
    for (LVIMSession *session in arrM) {
        NSMutableDictionary *sessionDict = [self msgToDict:session.lastMsg];
        sessionDict[@"unreadCount"] = @(session.unreadCount);
        [msgs addObject:sessionDict];
    }
    dictM[@"sessions"] = msgs;
    result(dictM);
}

- (void)queryAllSessionList:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    NSMutableArray<LVIMSession *> *arrM = [NSMutableArray array];
    int num = [[LVIMSDK sharedInstance].conversationManager queryAllSessionList:arrM];
    
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    dictM[@"result"] = @(num);
    
    if (num <= 0) {
        result(dictM);
        return;
    }
    
    NSMutableArray *msgs = [NSMutableArray arrayWithCapacity:num];
    for (LVIMSession *session in arrM) {
        NSMutableDictionary *sessionDict = [self msgToDict:session.lastMsg];
        sessionDict[@"unreadCount"] = @(session.unreadCount);
        [msgs addObject:sessionDict];
    }
    dictM[@"sessions"] = msgs;
    result(dictM);
}

- (void)sendMessage:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int64_t cmsgid = [dict[@"cmsgid"] longLongValue];
    int sub = [dict[@"sub"] intValue];
    NSString *tid = dict[@"tid"];
    NSString *type = dict[@"type"];
    NSString *content = dict[@"content"];
    NSString *pushTitle = dict[@"pushTitle"];
    NSString *pushBody = dict[@"pushBody"];
    NSString *pushExtra = dict[@"pushExtra"];
    NSString *click_action = dict[@"pushClickAction"];
    
    LVPushContent *push;
    if (pushTitle || pushBody || pushExtra || click_action) {
        push = [LVPushContent new];
        push.title = pushTitle;
        push.body = pushBody;
        push.extra = pushExtra;
        push.click_action = click_action;
    }
    
    LVIMMessage *msg = [LVIMMessage new];
    int rs = [msg buildPrivateRequest:sub fid:self.uid tid:tid type:type content:content pushContent:push extend3:nil extend4:nil targetAppID:nil targetAppUID:nil];
    msg.mLVsgID = cmsgid;
    // 构建消息失败
    if (rs != 0) {
        result(@{@"eCode": @(rs)});
        return;
    }
    
    // 构建消息成功
    int send_rs = [[LVIMSDK sharedInstance] sendMessage:msg context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(ecode);
        dictM[@"rcode"] = @(rcode);
        result(dictM);
    }];
    
    // 发送消息失败
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [self msgToDict:msg];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }
}

- (void)setPrivateDBStorageMax:(FlutterMethodCall *)call result:(FlutterResult)result {
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    
    int64_t max = [dict[@"max"] longLongValue];
    if (max <= 0) {
        result(@(-1));
        return;
    }
    
    int rs = [[LVIMSDK sharedInstance].msgManager setDBStorageMax:(LVIMDB_TYPE_PRIVATE) storageMax:max];
    result(@(rs));
}

- (void)queryUnReadMessageNum:(FlutterMethodCall *)call result:(FlutterResult)result {
    int count = [[LVIMSDK sharedInstance].conversationManager queryUnReadCount];
    result(@(count));
}

- (void)uploadPushToken:(FlutterMethodCall *)call result:(FlutterResult)result {
    
    if (![call.arguments isKindOfClass:[NSDictionary class]]) return;
    NSDictionary *dict = [self removeNull:call.arguments];
    NSString *token = dict[@"token"];
    
    [[LVIMSDK sharedInstance] updatePushToken:token complete:^(int code, NSString *msg) {
        NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
        dictM[@"eCode"] = @(code);
        dictM[@"msg"] = msg;
        
        result(dictM);
    }];
}

#pragma mark - RoomCallbackProtocl

- (void)onAddRemoterUser:(NSString *)uid {
    [self.channel invokeMethod:@"onRemoteStreamAdd" arguments:@{@"userId": uid ?: @""}];
}
- (void)onRemoteLeave:(NSString *)uid {
    [self.channel invokeMethod:@"onRemoteStreamEnd" arguments:@{@"userId": uid ?: @""}];
}

/**
 * 被踢出房间或断开房间链接
 * @param errorCode 错误码，参见Zego或LinkV错误码
 * @param roomId 房间id
 */
- (void)onRoomDisconnect:(int)errorCode roomId:(NSString *)roomId {
    NSLog(@"[Wing] onRoomDisconnect roomId = %@ errorCode = %d", roomId, errorCode);
    [self.channel invokeMethod:@"onRoomDisconnect" arguments:@{@"errorCode": @(errorCode), @"roomID": roomId ?: @""}];
}

- (void)onRoomConnected:(NSString *)roomId {
    NSLog(@"[Wing] onRoomConnected roomId = %@", roomId);
    [self.channel invokeMethod:@"onRoomConnected" arguments:@{@"roomID": roomId ?: @""}];
}
- (void)onVideoSizeChanged:(NSString *)uid width:(int)width height:(int)height {
    [self.channel invokeMethod:@"onVideoSizeChanged" arguments:@{@"streamID": uid, @"width": @(width), @"height": @(height)}];
}
- (void)onPublishStreamQualityUpdate:(NSString *)uid quality:(VideoQuality *)quality {
    [self.channel invokeMethod:@"onPublisherQualityUpdate" arguments:@{@"videoSentPackets": @(quality.videoSentPackets), @"videoSentKBytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videoBitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight)}];
}
- (void)onRemoteQualityUpdate:(NSString *)uid quality:(VideoQuality *)quality {
    [self.channel invokeMethod:@"onRemoteQualityUpdate" arguments:@{@"streamID": uid, @"videoSentPackets": @(quality.videoSentPackets), @"videoSentKBytes": @(quality.videosentKbytes), @"videoFps": @(quality.videoFps), @"videoLostPackets": @(quality.videoLostPackets), @"videoBitrateBps": @(quality.videoBitratebps), @"videoRtt": @(quality.videoRtt), @"videoLostPercent": @(quality.videoLostPercent), @"frameWidth": @(quality.frameWidth), @"frameHeight": @(quality.frameHeight)}];
}

/**
 * 鉴权回调
 * @param errorCode 0表示鉴权成功，其余均为错误，参见Zego或LinkV错误码
 */
- (void)authComplete:(NSInteger)errorCode {
    if (!self.initResult) return;
    
    NSLog(@"[Wing] authComplete errorCode = %@  imCode = %@", @(errorCode),@(self.imStatus));
    self.initResult(@{@"rtcCode": @(errorCode), @"imCode": @(self.imStatus?1:0)});

    self.initResult = nil;
}

- (void)onPlayStateUpdate:(int)code userId:(nonnull NSString *)userId {
    [self.channel invokeMethod:@"onPlayStateUpdate" arguments:@{@"streamId": userId, @"state": @(code)}];
}
- (void)onPublishStateUpdate:(int)code {
    [self.channel invokeMethod:@"onPublishStateUpdate" arguments:@{@"state": @(code)}];
}

/// 收到房间消息
/// 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
- (int)onRoomMessageReceive:(LVIMMessage *)msg {
    [self.channel invokeMethod:@"onRoomMessageReceive" arguments:[self msgToDict:msg]];
    
    return 0;
}

#pragma mark - LVIMModuleEventDelegate
/**
 @brief 请求token的事件
 **/
-(void)onQueryIMToken {
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onQueryIMToken" arguments:nil];
    });
}

/**
 @brief 连接成功事件
 @param context 状态信息
 **/
-(void)onIMConnected:(void*)context {
    NSLog(@"[Wing] onIMConnected context = %@", context);
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMConnected" arguments:nil];
    });

}

/**
 @brief 连接断开事件
 **/
-(void)onIMLosted {
    NSLog(@"[Wing] onIMLosted");
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMLosted" arguments:nil];
    });
}

/**
 @brief 认证成功事件
 @param uid 用户ID
 @param token token (IM)
 @param unReadMsgSize 未读消息数量(粗略)
 **/
-(void)onIMAuthSuccessed:(NSString*)uid token:(NSString*)token unReadMsgSize:(int)unReadMsgSize {
    NSLog(@"[Wing] onIMAuthSuccessed uid = %@, token = %@", uid, token);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMAuthSucceed" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"unReadMsgSize": @(unReadMsgSize)}];
    });
}

/**
 @brief 认证失败事件
 @param uid 用户ID
 @param token 认证码(im)
 @param ecode 错误码
 @param rcode 响应码
 @param expired 是否是过期了
 **/
-(void)onIMAuthFailed:(NSString*)uid token:(NSString*)token ecode:(int)ecode rcode:(int)rcode expired:(BOOL)expired {
    NSLog(@"[Wing] onIMAuthFailed uid = %@, token = %@, ecode = %@, rcode = %@, expired = %d", uid, token, @(ecode), @(rcode), expired);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMAuthFailed" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"ecode": @(ecode), @"rcode": @(rcode), @"isTokenExpired": @(expired)}];
    });
}

/**
 @brief 认证过期
 @param uid 用户ID
 @param token 认证码(im)
 @param owner 所属者
 **/
-(void)onIMTokenExpired:(NSString*)uid token:(NSString*)token owner:(NSString*)owner {
    NSLog(@"[Wing] onIMTokenExpired uid = %@", uid);
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.channel invokeMethod:@"onIMTokenExpired" arguments:@{@"uid": uid ?: @"", @"token": token ?: @"", @"emsg": owner}];
    });
}

-(void)onIMUserKicked:(NSString *)uid code:(LVIM_LOGIN_KICKED_CODE)code
{
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    if (uid) {
        dict[@"uid"] = uid;
    }
    dict[@"code"] = @(code);
    [self.channel invokeMethod:@"onIMUserKicked" arguments:dict];
}

#pragma mark - LVIMReceiveMessageDelegate
/**
 @brief 消息过滤器
 @param diytype 消息类型的CRC32
 @param fromid 发送者ID
 @param toid 接收者ID
 @param msgtype 类型别名
 @param content 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 过滤丢弃掉这个消息返回YES, 继续处理返回NO
 **/
-(BOOL)onIMReceiveMessageFilter:(int32_t)diytype
                           fromid:(const char*)fromid
                             toid:(const char*)toid
                          msgtype:(const char*)msgtype
                          content:(const char*)content
                         waitings:(int)waitings
                       packetSize:(int)packetSize
                       waitLength:(int)waitLength
                     bufferSize:(int)bufferSize {
    
    return NO;
}

/**
 @brief 消息返回处理
 @param owner 这个消息属于哪个登陆用户
 @param msg 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 处理成功返回0, 否则返回非0
 @warning 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
 **/
-(int)onIMReceiveMessageHandler:(NSString*)owner
                          immsg:(LVIMMessage*)msg
                       waitings:(int)waitings
                     packetSize:(int)packetSize
                     waitLength:(int)waitLength
                     bufferSize:(int)bufferSize {

    [self.channel invokeMethod:@"onIMMessageReceive" arguments:[self msgToDict:msg]];
    
    return 0;
}

-(void)onIMMessageReadedAckWithMsgIDList:(NSArray<NSNumber *> *)idList
{
    [self.channel invokeMethod:@"onIMMessageReadedAcks" arguments:idList];
}

#pragma mark - private


/// 内部发送媒体消息
-(void)_sendMediaMessages:(id)mediaMessage error:(NSError *)error result:(FlutterResult)result{
    if (error) {
        NSMutableDictionary *dictM = [self msgToDict:mediaMessage];
        dictM[@"eCode"] = @(error.code);
        result(dictM);
        return;
    }
    // 构建消息成功
    int send_rs = [[LVIMSDK sharedInstance] sendMessage:mediaMessage context:nil callback:^(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg) {
        NSMutableDictionary *dictM = [self msgToDict:mediaMessage];
        dictM[@"eCode"] = @(ecode);
        dictM[@"rcode"] = @(rcode);
        result(dictM);
    }];
    
    // 发送消息失败
    if (send_rs != 0) {
        NSMutableDictionary *dictM = [self msgToDict:mediaMessage];
        dictM[@"eCode"] = @(send_rs);
        result(dictM);
    }
}


- (NSDictionary *)removeNull:(NSDictionary *)dict {
    NSArray *allKeys = dict.allKeys;
    
    NSMutableDictionary *dictM = [dict mutableCopy];
    for (NSString *key in allKeys) {
        if ([dict[key] isKindOfClass:[NSNull class]]) {
            [dictM removeObjectForKey:key];
        }
    }
    
    return dictM;
}

- (NSMutableDictionary *)msgToDict:(LVIMMessage *)msg {
    NSMutableDictionary *dictM = [NSMutableDictionary dictionary];
    if (!msg) return dictM;
    
    dictM[@"cmdType"] = @(msg.mCmdType);
    dictM[@"subType"] = @(msg.mSubType);
    dictM[@"fromID"] = msg.mFromID;
    dictM[@"toID"] = msg.mToID;
    dictM[@"msgType"] = msg.mExtend1;
    dictM[@"extend2"] = msg.mExtend2;
    dictM[@"extend3"] = msg.mExtend3;
    dictM[@"extend4"] = msg.mExtend4;
    dictM[@"extend5"] = msg.mExtend5;
    dictM[@"extend6"] = msg.mExtend6;
    dictM[@"pushTitle"] = msg.mPushTitle;
    dictM[@"cmsgid"] = @(msg.mLVsgID);
    dictM[@"smsgid"] = @(msg.mSMsgID);
    dictM[@"dbid"] = @(msg.mDBID);
    dictM[@"ctime"] = @(msg.mClientTimestamp);
    dictM[@"stime"] = @(msg.mServerTimestamp);
    dictM[@"origin"] = @(0);
    dictM[@"msgContent"] = msg.mContent;
    dictM[@"pushTitle"] = msg.mPushTitle;
    dictM[@"sendState"] = @(msg.send_state);
    dictM[@"sequence"] = @(msg.mSequence);
    
    if (msg.mSubType == LVIM_MESSAGE_SUBTYPE_AUDIO) {//音频

     
        LVIMAudioMessage *audioMsg = (LVIMAudioMessage *)msg;
        dictM[@"needSaveToDB"] = @(1);
        dictM[@"audioData"] = audioMsg.audioData;
        dictM[@"duration"] = [NSString stringWithFormat:@"%.2f",audioMsg.duration];
        
        NSMutableDictionary *audioFile = [NSMutableDictionary dictionary];
        audioFile[@"key"] = audioMsg.audioFile.key;
        audioFile[@"localPath"] = audioMsg.audioFile.localPath;
        audioFile[@"pathExtension"] = audioMsg.audioFile.pathExtension;
        audioFile[@"fileLength"] = @(audioMsg.audioFile.fileLength);
        audioFile[@"filetype"] = @(audioMsg.audioFile.filetype);
        dictM[@"audioFile"] = audioFile;
  
    }else if (msg.mSubType == LVIM_MESSAGE_SUBTYPE_IMAGE){//图片
   
        LVIMImageMessage *imageMsg = (LVIMImageMessage *)msg;
        dictM[@"needSaveToDB"] = @(1);
        dictM[@"image"] = @"";
        dictM[@"imageSize"] = NSStringFromCGSize(imageMsg.imageSize);
        dictM[@"thumbnailImage"] = @"";
        dictM[@"thumbnailImageSize"] = NSStringFromCGSize(imageMsg.thumbnailImageSize);
        dictM[@"isOrigin"] = @(imageMsg.isOrigin);
        
        NSMutableDictionary *imageFile = [NSMutableDictionary dictionary];
        imageFile[@"key"] = imageMsg.imageFile.key;
        imageFile[@"localPath"] = imageMsg.imageFile.localPath;
        imageFile[@"pathExtension"] = imageMsg.imageFile.pathExtension;
        imageFile[@"fileLength"] = @(imageMsg.imageFile.fileLength);
        imageFile[@"filetype"] = @(imageMsg.imageFile.filetype);
        dictM[@"imageFile"] = imageFile;
        
        NSMutableDictionary *thumbnailImageFile = [NSMutableDictionary dictionary];
        thumbnailImageFile[@"key"] = imageMsg.thumbnailImageFile.key;
        thumbnailImageFile[@"localPath"] = imageMsg.thumbnailImageFile.localPath;
        thumbnailImageFile[@"pathExtension"] = imageMsg.thumbnailImageFile.pathExtension;
        thumbnailImageFile[@"fileLength"] = @(imageMsg.thumbnailImageFile.fileLength);
        thumbnailImageFile[@"filetype"] = @(imageMsg.thumbnailImageFile.filetype);
        dictM[@"thumbnailImageFile"] = thumbnailImageFile;
   
    }else if (msg.mSubType == LVIM_MESSAGE_SUBTYPE_VIDEO){//视频
    
        LVIMVideoMessage *videoMsg = (LVIMVideoMessage *)msg;
        dictM[@"needSaveToDB"] = @(1);
        dictM[@"videoSize"] = NSStringFromCGSize(videoMsg.videoSize);
        dictM[@"duration"] = [NSString stringWithFormat:@"%.2f",videoMsg.duration];
        dictM[@"image"] = @"";
        dictM[@"imageSize"] = NSStringFromCGSize(videoMsg.imageSize);
        dictM[@"thumbnailImage"] = @"";
        dictM[@"thumbnailImageSize"] = NSStringFromCGSize(videoMsg.thumbnailImageSize);

        NSMutableDictionary *videoFile = [NSMutableDictionary dictionary];
        videoFile[@"key"] = videoMsg.videoFile.key;
        videoFile[@"localPath"] = videoMsg.videoFile.localPath;
        videoFile[@"pathExtension"] = videoMsg.videoFile.pathExtension;
        videoFile[@"fileLength"] = @(videoMsg.videoFile.fileLength);
        videoFile[@"filetype"] = @(videoMsg.videoFile.filetype);
        dictM[@"videoFile"] = videoFile;
        
        NSMutableDictionary *imageFile = [NSMutableDictionary dictionary];
        imageFile[@"key"] = videoMsg.imageFile.key;
        imageFile[@"localPath"] = videoMsg.imageFile.localPath;
        imageFile[@"pathExtension"] = videoMsg.imageFile.pathExtension;
        imageFile[@"fileLength"] = @(videoMsg.imageFile.fileLength);
        imageFile[@"filetype"] = @(videoMsg.imageFile.filetype);
        dictM[@"imageFile"] = imageFile;

        NSMutableDictionary *thumbnailImageFile = [NSMutableDictionary dictionary];
        thumbnailImageFile[@"key"] = videoMsg.thumbnailImageFile.key;
        thumbnailImageFile[@"localPath"] = videoMsg.thumbnailImageFile.localPath;
        thumbnailImageFile[@"pathExtension"] = videoMsg.thumbnailImageFile.pathExtension;
        thumbnailImageFile[@"fileLength"] = @(videoMsg.thumbnailImageFile.fileLength);
        thumbnailImageFile[@"filetype"] = @(videoMsg.thumbnailImageFile.filetype);
        dictM[@"thumbnailImageFile"] = thumbnailImageFile;
   
    }

    return dictM;
}

#pragma mark - public
+ (int)uploadPushToken:(NSData *)deviceToken complete:(void(^)(int code, NSString *msg))complete {
    if (!deviceToken) return -1;
    
    NSMutableString *token = [NSMutableString string];
    const char *bytes = (char *)[deviceToken bytes];
    NSInteger count = deviceToken.length;
    for (int i = 0; i < count; i++) {
        [token appendFormat:@"%02x", bytes[i]&0x000000FF];
    }
    
    return [[LVIMSDK sharedInstance] updatePushToken:token complete:complete];
}


@end
