//
//  LVIMChatroomManager.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/16.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "LVIMEnumDefine.h"
#import "LVIMRoomState.h"

NS_ASSUME_NONNULL_BEGIN

@interface LVIMChatroomManager : NSObject

/**
 @brief 创建房间
 @param rid 房间ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)createChatroom:(NSString*)rid
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 加入房间
 @param rid 房间ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)joinChatroom:(NSString*)rid
           context:(id)context
          callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 离开房间
 @param rid 房间ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)leaveChatroom:(NSString*)rid
            context:(id)context
           callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 房间用户拉黑
 @param rid 房间ID
 @param blackid 拉黑用户ID
 @param isAdd 添加或者删除
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)chatroomBlack:(NSString*)rid
            blackUser:(NSString *)blackid
            isAdd:(BOOL)isAdd
            context:(id)context
           callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;


/**
 @brief 房间用户禁言
 @param rid 房间ID
 @param gagid 禁言用户ID
 @param isAdd 添加或者删除
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)chatroomGag:(NSString*)rid
            gagUser:(NSString *)gagid
             time:(int)time
            isAdd:(BOOL)isAdd
            context:(id)context
         callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 房间附加数据
 @param rid 房间ID
 @param opType 操作方式/增删改查
 @param key key
 @param value value
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)chatroomData:(NSString*)rid
            opType:(LVIM_ROOM_DATA_TYPE)opType
        key:(NSString *)key
        value:(NSString *)value
            context:(id)context
          callback:(LVIM_ROOM_OBJECT_CALLBACK)callback;

/**
 @brief 房间用户附加数据
 @param rid 房间ID
 @param opType 操作方式/增删改查
 @param key key
 @param value value
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)chatroomUserData:(NSString*)rid
            opType:(LVIM_ROOM_USER_DATA_TYPE)opType
        key:(NSString *)key
        value:(NSString *)value
            context:(id)context
              callback:(LVIM_ROOM_OBJECT_CALLBACK)callback;



/**
 @brief 获取当前房间的状态
 **/
-(LVIMRoomState*)getCurrentRoomState;

@end

NS_ASSUME_NONNULL_END
