//
//  LVIMConfig.h
//  LVIM
//
//  Created by pangbingxin on 2018/7/31.
//  Copyright © 2018年 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <LVIMLib/LVIMEnumDefine.h>
@class LVIMMessage;

//对应媒体文件的文件夹名称
extern NSString* g_lvim_media_file_directory_names[LVIM_MEDIA_FILE_TYPE_MAX];

//媒体配置信息
struct lvim_media_config {
    //图片文件最大字节数限制(最大1024KB)
    int     image_fsize_max;
    //图片最高质量(最小30,默认70,最大100)
    int     image_quality_max;
    //图片最小宽高度限制(默认4)
    int     image_size_min;
    //图片最大宽度限制(默认1024,超过会被裁减)
    int     image_size_max;
    
    //缩略图最高质量(最小30,默认70,最大100)
    int     thumb_quality_max;
    //缩略图最小宽高度限制(默认4)
    int     thumb_size_min;
    //缩略图最大宽度限制(默认64,超过会被裁减)
    int     thumb_size_max;
    
    //音频需要走上传的大小(低于这个值走im,高于这个值走上传,默认5KB)
    int     audio_size_upload_limit;
    //音频需要走上传的秒数(低于这个值走im,高于这个值走上传,默认5秒)
    int     audio_duration_upload_limit;
};
typedef struct lvim_media_config lvim_media_config;

@interface LVIMConfig : NSObject<NSCopying>
//----------------------------------------------------------------------
//应用ID
@property (nonatomic, copy) NSString* mAppID;
//应用KEY
@property (nonatomic, copy) NSString* mAppKey;
//应用AID
@property (nonatomic, copy) NSString* mAppXAID;
//应用的设备ID
@property (nonatomic, copy) NSString* mAppDeviceID;
//----------------------------------------------------------------------
//应用的用户ID(初始化可空)
@property (nonatomic, copy) NSString* mAppUserID;
//应用的用户国家码(初始化可空)
@property (nonatomic, copy) NSString* mAppUserCountry;
//----------------------------------------------------------------------
//获取IM的配置地址(正式)
@property (nonatomic, copy) NSString* mIMQueryConfigURL;
//获取IM的配置地址(测试)
@property (nonatomic, copy) NSString* mIMQueryConfigURL_DEBUG;
//----------------------------------------------------------------------
//获取IM的未读消息地址(正式)
@property (nonatomic, copy) NSString* mIMQueryUnReadMessageURL;
//获取IM的未读消息地址(测试)
@property (nonatomic, copy) NSString* mIMQueryUnReadMessageURL_DEBUG;
//----------------------------------------------------------------------
//获取IM的历史消息地址(正式)
@property (nonatomic, copy) NSString* mIMQuerySessionMessageURL;
//获取IM的历史消息地址(测试)
@property (nonatomic, copy) NSString* mIMQuerySessionMessageURL_DEBUG;
//----------------------------------------------------------------------
//获取IM的会话列表地址(正式)
@property (nonatomic, copy) NSString* mIMQuerySessionListURL;
//获取IM的会话列表地址(测试)
@property (nonatomic, copy) NSString* mIMQuerySessionListURL_DEBUG;
//----------------------------------------------------------------------
//获取上传IM的文件的TOKEN地址(正式)
@property (nonatomic, copy) NSString* mIMQueryUploadTokenURL;
//获取上传IM的文件的TOKEN地址(测试)
@property (nonatomic, copy) NSString* mIMQueryUploadTokenURL_DEBUG;

//获取新的上传IM的文件的TOKEN地址(正式)
@property (nonatomic, copy) NSString* mNewIMQueryUploadTokenURL;
//获取新的上传IM的文件的TOKEN地址(测试)
@property (nonatomic, copy) NSString* mNewIMQueryUploadTokenURL_DEBUG;
//----------------------------------------------------------------------
//IM的设备ID
@property (nonatomic, copy) NSString* mIMDeviceID;
//APNS推送TOKEN
@property (nonatomic, copy) NSString* mPushToken;
//----------------------------------------------------------------------
//私信媒体文件的配置信息
@property (nonatomic, assign) lvim_media_config mIMPrivateMediaConfig;
//群组媒体文件的配置信息
@property (nonatomic, assign) lvim_media_config mIMGroupMediaConfig;
//----------------------------------------------------------------------
//是否是调试模式(默认为关)
@property (nonatomic, assign) BOOL mIsDebugMode;
//是否显示日志(默认显示)
@property (nonatomic, assign) BOOL mIsLogVisible;
//是否启用派发线程(默认启用)
@property (nonatomic, assign) BOOL mIsDispatchThreadEnable;
//是否启用房间(默认直播间为开)
@property (nonatomic, assign) BOOL mIsChatroomEnabled;
//是否启用私信(默认私信为关)
@property (nonatomic, assign) BOOL mIsPrivateEnabled;
//是否启用群组(默认群组为关)
@property (nonatomic, assign) BOOL mIsGroupEnabled;
//是否允许记录连接信息和消息发送状态日志
@property (nonatomic, assign) BOOL mIsAllowRecordConnectionInfo;
//----------------------------------------------------------------------

/**
 @brief 检测内容是否无效
 @return 无效返回非0, 有效返回0
 **/
-(int)invalid;

/**
 @brief 判断是否登录成功
 **/
-(BOOL)isAppUserLoginSuccessed;

@end
