//
//  LVIMConversationManager.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/17.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "LVIMEnumDefine.h"
#import "LVIMProtocols.h"

@class LVIMSession;

NS_ASSUME_NONNULL_BEGIN

@interface LVIMConversationManager : NSObject

/**
 @brief 拉取远程历史会话列表
 @param pageIndex 页面编号
 @param pageSize 页面长度
 @param callback 结果回调
 @return 成功返回0, 否则返回非0
 **/
-(int)queryRemoteSessionList:(int)pageIndex pageSize:(int)pageSize callback:(LVIM_QUERY_REMOTE_SESSION_LIST_CALLBACK)callback;

/**
 @brief 把某个会话中的所有未读消息标记为已读
 @param targetId 群组是群组ID,私信是对方ID
 @param type 私信或者群组
 @return 成功返回处理条数, 否则返回<0的值
 **/
-(int)clearSessionUnreadMsg:(NSString *)targetId type:(LVIMDB_TABLE_TYPE)type;


/**
@brief 把某个会话中小于等于stime的所有未读消息标记为已读
@param targetId 目标的id
@param stime 消息的服务端时间戳
@param type 私信或者群组
@return 成功返回处理条数, 否则返回<0的值
*/
-(int)updateMsgAsReadByStime:(NSString *)targetId stime:(int64_t)stime type:(LVIMDB_TABLE_TYPE)type;

/**
@brief 把所有未读消息标记为已读
@return 成功返回处理条数, 否则返回<0的值
*/
-(int)clearAllUnreadMsg:(LVIMDB_TABLE_TYPE)type;

/**
@brief 获取消息会话列表
@param arrM 结果集
@param type 私信或者群组
@return 成功返回处理条数, 否则返回<0的值
*/
-(int)querySessionList:(NSMutableArray<LVIMSession *> *)arrM type:(LVIMDB_TABLE_TYPE)type;

/**
@brief 获取群和私信消息会话列表
@param arrM 结果集
@return 成功返回处理条数, 否则返回<0的值
*/
- (int)queryAllSessionList:(NSMutableArray *)arrM;

/**
 @brief 获取所有未读消息的数量
 @return 成功返回未读消息数量, 否则返回<0的值
 */
-(int)queryUnReadCount;

/**
 @brief 设置模块会话代理(可根据需要设置)
 **/
-(void)setConversationDelegate:(id<LVIMConversationDelegate>)delegate;

/**
 @brief 设置是否自动通知会话和消息未读数变更
 **/
-(void)setNotifySessionAndUnReadNumberChangedEnable:(BOOL)enable;

@end

NS_ASSUME_NONNULL_END
