//
//  LVIMEnumDefine.h
//  LVIMLib
//
//  Created by 鹿容 on 2021/9/9.
//  Copyright © 2021 cmcm. All rights reserved.
//

#ifndef LVIMEnumDefine_h
#define LVIMEnumDefine_h
#ifdef __OBJC__
#import <Foundation/Foundation.h>

//用户ID长度限制
const static int LVIM_USER_ID_MAX = 256;
//群组ID长度限制
const static int LVIM_GROUP_ID_MAX = 35;
//群组名称长度限制
const static int LVIM_GROUP_NAME_MAX = 64;
//昵称长度限制
const static int LVIM_GROUP_NICKNAME_MAX = 25;
//群备注长度限制
const static int LVIM_GROUP_TOPIC_MAX = 1024;
//直播间ID长度限制
const static int LVIM_CHATROOM_ID_MAX = 256;
//直播间数据key长度限制
const static int LVIM_CHATROOM_INFO_KEY_MAX = 50;
//直播间数据value长度限制
const static int LVIM_CHATROOM_INFO_VALUE_MAX = 500;
//文件大小限制
const static long long LVIM_MEDIA_FILE_MAX = 200 * 1024 * 1024;


/**
 @brief 进度回调
 **/
typedef void(^LVIM_PROGRESS_CALLBACK)(float progress);
/**
 @brief 下载回调
 **/
typedef void(^LVIM_DOWNLOAD_CALLBACK)(NSError *error);


/**
 @brief 拉取会话列表的回调
 @param ecode 错误码(无错误为0)
 @param rcode 响应码(成功为200)
 @param rstatus 状态码(成功为200)
 @param sessions 会话列表
 **/
typedef void(^LVIM_QUERY_REMOTE_SESSION_LIST_CALLBACK)(int32_t ecode, int32_t rcode, int32_t rstatus, NSArray* sessions);

/**
 @brief 拉取历史消息的回调
 @param ecode 错误码(无错误为0)
 @param rcode 响应码(成功为200)
 @param rstatus 状态码(成功为200)
 @param messages 聊天记录
 **/
typedef void(^LVIM_QUERY_REMOTE_SESSION_MESSAGE_CALLBACK)(int32_t ecode, int32_t rcode, int32_t rstatus, NSArray* messages);
#endif

#pragma -mark 登录方式

typedef NS_ENUM(int, LVIM_LOGIN_TYPE) {
    LVIM_LOGIN_TYPE_MANUAL = 0,//手动登录
    LVIM_LOGIN_TYPE_AUTO = 1//自动登录
};

typedef NS_ENUM(int,LVIM_LOGIN_KICKED_CODE) {
    LVIM_LOGIN_KICKED_CODE_KICKED = -5,//有新的设备登录账号
    LVIM_LOGIN_KICKED_CODE_BAN = -9//用户拉黑
};

#pragma -mark 操作的数据库类型
//操作的数据库类型
typedef NS_ENUM(int, LVIMDB_TABLE_TYPE) {
    LVIMDB_TYPE_MIN = 0,
    LVIMDB_TYPE_ROOM = 1,
    LVIMDB_TYPE_GROUP = 2 ,
    LVIMDB_TYPE_PRIVATE = 3,
    LVIMDB_TYPE_SYSTEM = 4,
    LVIMDB_TYPE_MAX
};

#pragma -mark 媒体文件类型
//媒体文件类型
typedef NS_ENUM(int, LVIM_MEDIA_FILE_TYPE) {
    //无效最小值
    LVIM_MEDIA_FILE_TYPE_MIN = 0,
    //图片文件(私信)
    LVIM_MEDIA_FILE_PRIVATE_IMAGE,
    //语音文件(私信)
    LVIM_MEDIA_FILE_PRIVATE_AUDIO,
    //视频文件(私信)
    LVIM_MEDIA_FILE_PRIVATE_VIDEO,
    //文件文件(私信)
    LVIM_MEDIA_FILE_PRIVATE_FILE,
    //图片文件(群组)
    LVIM_MEDIA_FILE_GROUP_IMAGE,
    //语音文件(群组)
    LVIM_MEDIA_FILE_GROUP_AUDIO,
    //视频文件(群组)
    LVIM_MEDIA_FILE_GROUP_VIDEO,
    //文件文件(群组)
    LVIM_MEDIA_FILE_GROUP_FILE,
    //无效最大值
    LVIM_MEDIA_FILE_TYPE_MAX
};

//图片文件类型
typedef NS_ENUM(int, LVIM_IMAGE_FILE_TYPE){
    LVIM_IMAGE_FILE_TYPE_JPEG = 1,
    LVIM_IMAGE_FILE_TYPE_PNG = 2
};

#pragma -mark 直播间数据管理

typedef NS_ENUM(int, LVIM_ROOM_DATA_TYPE) {
    LVIM_ROOM_DATA_ADD          = 0, //直播间数据添加
    LVIM_ROOM_DATA_UPDATE      = 1, //直播间数据修改
    LVIM_ROOM_DATA_REMOVE      = 2, //直播间数据删除
    LVIM_ROOM_DATA_QUERY       = 3, //查询直播间某一个数据
    LVIM_ROOM_DATA_QUERY_ALL   = 4 //查询直播间所有数据
};

typedef NS_ENUM(int, LVIM_ROOM_USER_DATA_TYPE) {
    LVIM_ROOM_USER_DATA_ADD    = 0, //直播间用户数据添加
    LVIM_ROOM_USER_DATA_UPDATE = 1, //直播间用户数据修改
    LVIM_ROOM_USER_DATA_REMOVE = 2, //直播间用户数据删除
    LVIM_ROOM_USER_DATA_QUERY  = 4, //查询直播间用户某一个数据
    LVIM_ROOM_USER_DATA_QUERY_ALL  = 5 //查询直播间用户所有数据
};
#ifdef __OBJC__
//直播间取消禁言,取消全部禁言名单
const static NSString *CHAT_ROOM_GAG_ALL;
//直播间取消拉黑,取消全部拉黑名单
const static NSString *CHAT_ROOM_BLACK_ALL;
#endif

#pragma -mark 群组
//群组操作错误代码,对应rcode
typedef NS_ENUM(int, LVIM_GROUP_ERROR_TYPE) {
    LVIM_GROUP_ERROR_TYPE_SUC = 0,//操作成功
    LVIM_GROUP_ERROR_TYPE_SAVE_INFO_FAILED = 3002,//保存群组信息失败
    LVIM_GROUP_ERROR_TYPE_SAVE_INFO_AND_USER_FAILED = 3003,//保存群组和人员关系失败
    LVIM_GROUP_ERROR_TYPE_SAVE_GROUPFAILED = 3004,//保存人员拥有的群组信息失败
    LVIM_GROUP_ERROR_TYPE_GROUP_NO_USERS = 3005,//群组中没有群成员
    LVIM_GROUP_ERROR_TYPE_NOT_SUPPORT = 3006,//暂时不支持的批量操作
    LVIM_GROUP_ERROR_TYPE_UPDATE_FAILED = 3007,//更新群组信息失败
    LVIM_GROUP_ERROR_TYPE_ADMIN_QUIT_FAILED = 3102,//管理员退出群组失败,管理员可以解散群不能退出群
    LVIM_GROUP_ERROR_TYPE_PARAMAS_FAILED = 3105,//参数错误
    LVIM_GROUP_ERROR_TYPE_GROUP_ID_REPEAD = 3106,//参数错误 groupId 重复
    LVIM_GROUP_ERROR_TYPE_GROUP_NOT_EXSIST = 3107,//群组不存在
    LVIM_GROUP_ERROR_TYPE_ONLY_ADMIN = 3108,//管理员才能操作
    LVIM_GROUP_ERROR_TYPE_USER_IS_NOT_IN_GROUP = 3109,//用户不在群里面
    LVIM_GROUP_ERROR_TYPE_IS_DISABLED = 3110,//群组已经被禁用
};

typedef NS_ENUM(int,LVIM_PRIVATE_BLACK_TYPE) {
    LVIM_PRIVATE_BLACK_PARAMS_FAILED = 5050, //用户黑名单 参数错误，请确认 toUserId 是否为空，超出长度
    LVIM_PRIVATE_BLACK_CREATE_FAILED = 5051, //用户黑名单，黑名间创建失败
    LVIM_PRIVATE_BLACK_DELETE_FAILED = 5052, //用户黑名单，黑名单删除失败
    LVIM_PRIVATE_BLACK_EXSIT = 5053, //用户黑名单，黑名称已存在
    LVIM_PRIVATE_BLACK_NOT_EXSIT = 5054, //用户黑名单，黑名单不存在
    LVIM_PRIVATE_BLACK_LIMIT_ERROR = 5055 //用户黑名单数量超出限制
};

typedef NS_ENUM(int,LVIM_SEND_MESSAGE_RESULT) {
    LVIM_SEND_MESSAGE_RESULT_USERBLACK = 5056 //用户黑名单，用户已经被拉黑（发送私信时返回）
};

#pragma -mark 消息
@class LVIMMessage;
@class LVIMGroup;

/**
 @brief 消息已读回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 **/
typedef void(^LVIM_SEND_MSG_READED_CALLBACK)(int ecode, int rcode);

/**
 @brief 发送消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param msg 发送消息的消息体
 **/
typedef void(^LVIM_SEND_MESSAGE_CALLBACK)(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMMessage *msg);

/**
 @brief 发送消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param dicationry 回调的内容
 **/
typedef void(^LVIM_ROOM_OBJECT_CALLBACK)(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSDictionary *dicationry);

/**
 @brief 发送消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param gid 群ID
 **/
typedef void(^LVIM_GROUP_STRING_CALLBACK)(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSString *gid);

/**
 @brief 发送消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param list 列表
 **/
typedef void(^LVIM_GROUP_ARRAY_CALLBACK)(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, NSArray *list);

/**
 @brief 发送消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param model 群ID
 **/
typedef void(^LVIM_GROUP_MODEL_CALLBACK)(int ecode, int rcode, int64_t lvsgid, int64_t smsgid, int64_t stime, LVIMGroup *model);

/**
 @brief 发送HTTP消息的回调
 @param ecode 错误代码(0表示无错误)
 @param rcode 服务端返回码
 @param scode 服务端错误码
 @param hcode HTTP错误码
 @param hdelay HTTP耗时
 @param sdelay 服务端整体耗时
 @param qdelay 客户端整体耗时
 @param lvsgid 客户端消息id
 @param smsgid 服务端消息id
 @param stime 发送时间戳(毫秒)
 @param response 响应内容
 **/
typedef void(^LVIM_SEND_HTTP_MESSAGE_CALLBACK)(int ecode, int rcode, int scode, int hcode, int64_t hdelay, int64_t sdelay, int64_t qdelay, int64_t lvsgid, int64_t smsgid, int64_t stime, const char* response);

//消息类型
typedef NS_ENUM(int, LVIM_MESSAGE_LVDTYPE) {
    //最小无效消息类型
    LVIM_MESSAGE_LVDTYPE_MIN = -1,
    //直播间消息
    LVIM_MESSAGE_LVDTYPE_ROOM = 20,
    //群组消息
    LVIM_MESSAGE_LVDTYPE_GROUP = 15,
    //群组消息
    LVIM_MESSAGE_LVDTYPE_GROUP_T = 15,
    //私信消息
    LVIM_MESSAGE_LVDTYPE_PRIVATE = 17,
    //推送消息
    LVIM_MESSAGE_LVDTYPE_PUSH = 12,
    //事件消息
    LVIM_MESSAGE_LVDTYPE_EVENT = 32,
    //公告消息
    LVIM_MESSAGE_LVDTYPE_OFFICIAL = 31,
    //最大无效消息类型
    LVIM_MESSAGE_LVDTYPE_MAX
};

//内容类型
typedef NS_ENUM(int, LVIM_MESSAGE_SUBTYPE) {
    //最小无效内容类型
    LVIM_MESSAGE_SUBTYPE_MIN = -1,
    //自定义文本内容类型
    LVIM_MESSAGE_SUBTYPE_TEXT = 0,
    //自定义图片内容类型
    LVIM_MESSAGE_SUBTYPE_IMAGE = 1,
    //自定义音频内容类型
    LVIM_MESSAGE_SUBTYPE_AUDIO = 2,
    //自定义视频内容类型
    LVIM_MESSAGE_SUBTYPE_VIDEO = 3,
    //自定义军团文本内容类型
    LVIM_MESSAGE_SUBTYPE_LEGION_TEXT = 5,
    //自定义军团图片内容类型
    LVIM_MESSAGE_SUBTYPE_LEGION_IMAGE = 6,
    //自定义军团音频内容类型
    LVIM_MESSAGE_SUBTYPE_LEGION_AUDIO = 7,
    //自定义军团视频内容类型
    LVIM_MESSAGE_SUBTYPE_LEGION_VIDEO = 8,
    //自定义文本内容群消息
    LVIM_MESSAGE_SUBTYPE_GROUP_CHAT = 22,
    //最大无效内容类型
    LVIM_MESSAGE_SUBTYPE_MAX
};


typedef NS_ENUM(int, LVIM_MESSAGE_SEND_STATE) {
    //消息发送中
    LVIM_MESSAGE_SENDING = 0,
    //消息发送失败
    LVIM_MESSAGE_FAILED = 1,
    //消息发送成功——对方未读
    LVIM_MESSAGE_SENT_SUCC_UNREAD = 2,
    //消息发送成功——对方已读
    LVIM_MESSAGE_SENT_SUCC_READ = 3,
    //消息已收到——自己未读
    LVIM_MESSAGE_RECEIVED_UNREAD = 4,
    //消息已收到——自己已读
    LVIM_MESSAGE_RECEIVED_READ = 5,
    //消息多媒体转码失败,暂时只有视频
    LVIM_MESSAGE_ENCODE_MEDIA_FAILED = 6,
    //消息上传多媒体文件失败
    LVIM_MESSAGE_MEDIA_FAILED = 7,
    //已读回执
    LVIM_MESSAGE_READED_ACK = 8,
};


#pragma -mark 返回值

//ecode
typedef NS_ENUM(int,LVIM_ECODE_RESULT_CODE) {
    //ecode返回值,SDK 业务操作成功
    LVIM_ECODE_RESULT_CODE_OK = 0,
    //ecode返回值,认证过期
    LVIM_ECODE_RESULT_CODE_AUTH_EX = 100,
    //ecode返回值,请求或响应被取消丢弃
    LVIM_ECODE_RESULT_CODE_REQUEST_DROPED = 200,
    //ecode返回值,请求错误(发送失败)
    LVIM_ECODE_RESULT_CODE_REQUEST_ERROR = 300,
    //ecode返回值,请求超时
    LVIM_ECODE_RESULT_CODE_REQUEST_TIMEOUT = 301,
    //ecode返回值,等待响应错误(连接断开接收失败)
    LVIM_ECODE_RESULT_CODE_RESPONSE_ERROR = 400,
    //ecode返回值,等待响应超时
    LVIM_ECODE_RESULT_CODE_RESPONSE_TIMEOUT = 401
};

//函数返回值
typedef NS_ENUM(int, LVIM_FUNCTION_RESULT_CODE) {
    //函数调用成功
    LVIM_FUNCTION_RESULT_CODE_OK = 0,
    //参数错误
    LVIM_FUNCTION_RESULT_CODE_PARAMS_ERROR = 1,
    //登录用户ID为空
    LVIM_FUNCTION_RESULT_CODE_NO_UID = 2,
    //SDK 配置错误
    LVIM_FUNCTION_RESULT_CODE_CONFIG_ERROR = 3,
    //连接错误
    LVIM_FUNCTION_RESULT_CODE_CONNECTION_ERROR = 4,
    //请求错误
    LVIM_FUNCTION_RESULT_CODE_REQUEST_ERROR = 5,
    //未登录
    LVIM_FUNCTION_RESULT_CODE_LOGIN_ERROR = 6
    
};
//

#endif /* LVIMEnumDefine_h */
