//
//  LVIMFileUtils.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/23.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "LVIMEnumDefine.h"
#import <UIKit/UIKit.h>

@class LVIMFileContent;
NS_ASSUME_NONNULL_BEGIN

@interface LVIMFileUtils : NSObject

/**
 @brief 获取缓存的日志路径
 **/
+(NSString*)getCacheLogPath;

/**
 @brief 获取缓存的数据库路径
 **/
+(NSString*)getCacheDatabasePath;

/**
 @brief 获取随机缓存的媒体文件名称
 @param fremote 是否是远程下载的文件
 @param mftype 媒体类型(LVIM_MEDIA_FILE_TYPE)
 @param fid 发送者ID
 @param tid 接收者ID
 @param fsuffix 文件后缀
 **/
+(NSString*)getCacheMediaFileName:(BOOL)fremote mftype:(LVIM_MEDIA_FILE_TYPE)mftype fid:(NSString*)fid tid:(NSString*)tid fsuffix:(NSString*)fsuffix;

/**
 @brief 根据文件的key获取文件本地路径
 **/
+(NSString *)localPathWithFileContent:(LVIMFileContent *)fileContent;

/**
 @brief 获取文件大小
 **/
+ (float) getFileSize:(NSString *)path;

/**
 @brief 获取文件的时间
 **/
+ (float)getVideoLength:(NSURL *)URL;
/**
 @brief 读取文件数据
 **/
+(NSData*)loadFileData:(NSString*)fname ;
/**
 @brief 音频格式转换
 **/
+(NSData*)amrToWave:(NSData*)amr;
/**
 @brief 音频格式转换
 **/
+(NSData*)wavToamr:(NSData*)wav;

/**
 @brief 获取缩放大小
 **/
+ (CGSize)scaleSize:(CGSize)originSize maxSize:(CGSize)maxSize;
/**
 @brief 按固定高度缩放图片
 **/
+ (UIImage *)scaleImage:(UIImage *)image withFixedHeight:(CGFloat)fixedHeight;

+ (UIImage *)scaleImage:(UIImage *)image
                maxSize:(CGSize)maxSize;

/**
 @brief 讲图片存储到本地目录
 **/
+ (BOOL)saveImageFromName:(NSString *)imagePath image:(UIImage *)image;


/**
 @brief 视频转码,默认转换成960x540的mp4文件
 **/
+(BOOL)transVideoWithPath:(NSString *)videoPath completionHandler:(nonnull void (^)(BOOL,NSString *))handler;

/**
 @brief 获取视频的尺寸
 **/
+(CGSize)videoSize:(NSString *)videoPath;

/**
 @brief 获取视频首帧图片
 **/
+(UIImage *)getVideoFirstImage:(NSString *)videoPath;

@end

NS_ASSUME_NONNULL_END
