//
//  LVIMGroupManager.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/10.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "LVIMEnumDefine.h"

NS_ASSUME_NONNULL_BEGIN

@interface LVIMGroupManager : NSObject


/**
 @brief 创建群
 @param gid 群ID,可为空,服务器自动生成
 @param gname 群名称
 @param nickname 用户昵称
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)createGroup:(NSString*)gid
             gname:(NSString *)gname
         nickname:(NSString *)nickname
             context:(id)context
            callback:(LVIM_GROUP_STRING_CALLBACK)callback;

/**
 @brief 加入群
 @param gid 群ID
 @param nickname 用户昵称
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)joinGroup:(NSString*)gid
         nickname:(NSString *)nickname
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 获取群的信息
 @param gid 群ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)getGroupInfo:(NSString*)gid
             context:(id)context
            callback:(LVIM_GROUP_MODEL_CALLBACK)callback;

/**
 @brief 更新群名称
 @param gid 群ID
 @param gname 群名称
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)updateGroupName:(NSString*)gid
            gname:(NSString *)gname
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 转让管理员
 @param gid 群ID
 @param touid 新管理员的ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)changeGroupAdmin:(NSString*)gid
            touid:(NSString *)touid
             context:(id)context
             callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 获取群用户列表
 @param gid 群ID
 @param page 页数,从1开始
 @param size 每页的数量
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)getGroupUserList:(NSString*)gid
                  page:(int)page
                  size:(int)size
             context:(id)context
              callback:(LVIM_GROUP_ARRAY_CALLBACK)callback;

/**
 @brief 退群
 @param gid 群ID
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)leaveGroup:(NSString*)gid
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;


/**
 @brief 获取当前用户所在的群列表
 @param page 页数,由1开始
 @param size 每页的数量
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)getGroupList:(int)page
             size:(int)size
           context:(id)context
            callback:(LVIM_GROUP_ARRAY_CALLBACK)callback;

/**
 @brief 解散群
 @param gid 群ID
 @param block ,值为1是是解散群
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)operateGroup:(NSString*)gid
             block:(int)block
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 编辑群备注
 @param gid 群ID
 @param topic 群备注
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)editGroupTopic:(NSString*)gid
              topic:(NSString *)topic
             context:(id)context
            callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;


@end

NS_ASSUME_NONNULL_END
