//
//  LVIMMsgManage.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/9.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <LVIMLib/LVIMMessage.h>
#import <LVIMLib/LVIMEnumDefine.h>
#import "LVIMEnumDefine.h"

NS_ASSUME_NONNULL_BEGIN

@interface LVIMMsgManager : NSObject

/**
 @brief 删除本地私信历史消息记录
 @param sessionid 某个人的UID
 @return 成功返回删除条数, 失败返回<0的值
 **/
-(int)deleteLocalPrivateHistoryMessage:(NSString*)sessionid;

/**
 @brief 删除本地私信历史消息记录
 @param sessionid 某个群组的ID
 @return 成功返回删除条数, 失败返回<0的值
 **/
-(int)deleteLocalGroupHistoryMessage:(NSString*)sessionid;


/**
 @brief 查询本地私信消息历史记录
 @param sessionid 某个人的UID
 @param dbid 参考数据id
 @param limit 限制结果条数
 @param desc 是否降序查询
 @param rmsgs 结果集
 @return 成功返回结果数量, 失败返回<0的值
 **/
-(int)queryLocalPrivateHistoryMessage:(NSString*)sessionid dbid:(int64_t)dbid limit:(int32_t)limit desc:(BOOL)desc rmsgs:(NSMutableArray<LVIMMessage *> *)rmsgs;

/**
 @brief 查询本地群组消息历史记录
 @param sessionid 某个群组的ID
 @param dbid 参考数据id
 @param limit 限制结果条数
 @param desc 是否降序查询
 @param rmsgs 结果集
 @return 成功返回结果数量, 失败返回<0的值
 **/
-(int)queryLocalGroupHistoryMessage:(NSString*)sessionid dbid:(int64_t)dbid limit:(int32_t)limit desc:(BOOL)desc rmsgs:(NSMutableArray*)rmsgs;


/**
 @brief 拉取远程某个会话的历史消息
 @param tuid 目标会话ID
 @param sequence 本地最后一条数据的服务端消息序列
 @param type 私信还是群组消息
 @param size 要获取的条数
 @param callback 结果回调
 @return 成功返回0, 否则返回非0
 **/
- (int)queryRemoteSessionMessage:(NSString *)tuid sequence:(int64_t)sequence size:(int)size type:(LVIM_MESSAGE_LVDTYPE)type callback:(LVIM_QUERY_REMOTE_SESSION_MESSAGE_CALLBACK)callback;

/**
@brief 设置消息的最大存储量
@param dbtype 数据库类型
@param storageMax 消息最大存储量
*/
-(int)setDBStorageMax:(LVIMDB_TABLE_TYPE)dbtype storageMax:(int64_t)storageMax;

/**
@brief 往数据库中插入某条消息
@param dbtype 数据库类型
@param msg        消息模型，必须有消息id，mLVsgID必须大于0
*/
- (int64_t)insertSendedMessageToDB:(LVIMDB_TABLE_TYPE)dbtype message:(LVIMMessage *)msg;

/**
 @brief 更新多媒体消息的状态,extend3字段
 如果是自定义消息,需要在发送前, 转码成功/失败, 上传成功/失败, 发送消息成功/失败 时分别调用
 @param dbtype 数据库类型
 @param msg        消息模型，必须有消息id，mLVsgID必须大于0
 **/
- (int64_t)updateSendedMediaMessageFromDB:(LVIMDB_TABLE_TYPE)dbtype message:(LVIMMessage *)msg;

/**
 @brief 标记消息已读
 @param msgList 消息列表
 **/
-(int64_t)sendPrivateMessageReadedAckWithMessageList:(NSArray<LVIMMessage *> *)msgList sessionid:(NSString *)sessionid context:(id)context callBack:(LVIM_SEND_MSG_READED_CALLBACK)callBack;

@end

NS_ASSUME_NONNULL_END
