//
//  LVIMProtocols.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/22.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN
//会话刷新代理
@protocol LVIMConversationDelegate <NSObject>

@optional
/**
 @brief 会话列表更新通知
 **/
-(void)onConversationRefreshed;
/**
 @brief 总未读消息数更新通知
 **/
-(void)onAllUnReadNumberRefreshed;

@end


//定时刷新消息的代理
@protocol LVIMReceiveMessageFlushDelegate <NSObject>
@optional
/**
 @brief 刷新处理
 @param waitings 队列中等待处理的消息数量
 **/
-(void)onIMReceiveMessageFlush:(int)waitings;
@end

//返回消息处理代理
@protocol LVIMReceiveMessageDelegate <NSObject>
@required
/**
 @brief 消息返回处理
 @param owner 这个消息属于哪个登陆用户
 @param immsg 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 处理成功返回0, 否则返回非0
 @warning 如果返回非0, 则这条消息会在下次冷启动或再次检测到时继续调用本方法
 **/
-(int)onIMReceiveMessageHandler:(NSString*)owner
                          immsg:(LVIMMessage*)immsg
                       waitings:(int)waitings
                     packetSize:(int)packetSize
                     waitLength:(int)waitLength
                     bufferSize:(int)bufferSize;
@optional
/**
 @brief 消息过滤器
 @param diytype 消息类型的CRC32
 @param fromid 发送者ID
 @param toid 接收者ID
 @param msgtype 类型别名
 @param content 消息内容
 @param waitings 队列中等待处理的消息数量
 @param packetSize 当前数据包大小
 @param waitLength 等待处理的大小
 @param bufferSize 缓冲区最大大小
 @return 过滤丢弃掉这个消息返回YES, 继续处理返回NO
 **/
-(BOOL)onIMReceiveMessageFilter:(int32_t)diytype
                           fromid:(const char*)fromid
                             toid:(const char*)toid
                          msgtype:(const char*)msgtype
                          content:(const char*)content
                         waitings:(int)waitings
                       packetSize:(int)packetSize
                       waitLength:(int)waitLength
                       bufferSize:(int)bufferSize;
/**
 @brief 已读回执通知
 @param idList 消息ID列表
 **/
-(void)onIMMessageReadedAckWithMsgIDList:(NSArray<NSNumber *> *)idList;
@end

//信息统计回报代理
@protocol LVIMReportDelegate <NSObject>
@optional
/**
 @brief 回报连接状态
 @param state 状态信息
 **/
-(void)onIMReportConnectState:(void*)state;

/**
 @brief 回报消息状态
 @param state 状态信息
 **/
-(void)onIMReportMessageState:(void*)state;

@end



//事件代理
@protocol LVIMModuleEventDelegate <NSObject>
@required
/**
 @brief 请求token的事件
 **/
-(void)onQueryIMToken;
@optional
/**
 @brief 连接成功事件(暂不提供)
 @param context 状态信息
 **/
-(void)onIMConnected:(void*)context;

/**
 @brief 连接断开事件(暂不提供)
 **/
-(void)onIMLosted;

/**
 @brief 认证成功事件
 @param uid 用户ID
 @param token token (IM)
 @param unReadMsgSize 未读消息数量(粗略)
 **/
-(void)onIMAuthSuccessed:(NSString*)uid token:(NSString*)token unReadMsgSize:(int)unReadMsgSize;

/**
 @brief 认证失败事件
 @param uid 用户ID
 @param token 认证码(im)
 @param ecode 错误码
 @param rcode 响应码
 @param expired 是否是过期了
 **/
-(void)onIMAuthFailed:(NSString*)uid token:(NSString*)token ecode:(int)ecode rcode:(int)rcode expired:(BOOL)expired;

/**
 @brief 认证过期
 @param uid 用户ID
 @param token 认证码(im)
 **/
-(void)onIMTokenExpired:(NSString*)uid token:(NSString*)token;

/**
 @brief 用户被踢回调
 @param uid 用户ID
 **/
-(void)onIMUserKicked:(NSString *)uid code:(LVIM_LOGIN_KICKED_CODE)code;

@end

//上传文件的代理
@protocol LVIMUploadLogFilterDelegate <NSObject>
@optional
/**
 @brief 上传日志文件夹的代理
 @param content 携带参数
 @return 返回需要上传的文件夹路径列表
 **/
-(NSDictionary*)onIMUploadLogFilterDirectory:(NSString*)content;
/**
 @brief 上传日志文件的代理
 @param content 携带参数
 @return 返回需要上传的文件路径列表
 **/
-(NSDictionary*)onIMUploadLogFilterFile:(NSString*)content;
@end

NS_ASSUME_NONNULL_END
