//
//  LVIMSDK.h
//  LVIM
//
//  Created by pangbingxin on 2018/7/31.
//  Copyright © 2018年 cmcm. All rights reserved.
//

#import <LVIMLib/LVIMConfig.h>
#import <LVIMLib/LVIMRoomState.h>
#import <LVIMLib/LVIMMessage.h>
#import <LVIMLib/LVIMSession.h>
#import <LVIMLib/LVIMUser.h>
#import <LVIMLib/LVIMGroup.h>
#import <LVIMLib/LVIMProtocols.h>



//----------------------------------------------------------------------------------------------------

#define LVIM_WRITE_DEBUG_LOG(__format, ...) \
    lvim_write_debug_log([NSString stringWithFormat:__format, ##__VA_ARGS__])
FOUNDATION_EXPORT void lvim_write_debug_log(NSString* msg);

#define LVIM_WRITE_WARNING_LOG(__format, ...) \
    lvim_write_warning_log([NSString stringWithFormat:__format, ##__VA_ARGS__])
FOUNDATION_EXPORT void lvim_write_warning_log(NSString* msg);

#define LVIM_WRITE_ERROR_LOG(__format, ...) \
    lvim_write_error_log([NSString stringWithFormat:__format, ##__VA_ARGS__])
FOUNDATION_EXPORT void lvim_write_error_log(NSString* msg);

//----------------------------------------------------------------------------------------------------

//开启缩略图BASE64传输模式
//#define LVIM_IMAGE_THUMB_EXTRA_BASE64_ENABLE

//----------------------------------------------------------------------------------------------------
//是否开启自研im模块(关了它就全失效)
#define LVIM_MODULE_ENABLE

//是否开启自研im的私信
#define LVIM_PRIVATE_MODULE_ENABLE

//是否开启自颜im的群组
#define LVIM_GROUP_MODULE_ENABLE

//是否使用魔方控制(打开后使用魔方控制,关闭后强制开启)
#define LVIM_MODULE_USING_MOFANG

//是否使用魔方来控制日志打印(如果关掉它,强制打印日志)
#define LVIM_LOG_USING_MOFANG

//是否使用私信魔方控制(打开后使用魔方控制,关闭后强制开启)
#define LVIM_PRIVATE_USING_MOFANG

//是否使用群组魔方控制(打开后使用魔方控制,关闭后强制开启)
#define LVIM_GROUP_USING_MOFANG

//----------------------------------------------------------------------------------------------------

//检测房间是否已启用
#ifndef IS_LVIM_CHATROOM_ENABLE
#define IS_LVIM_CHATROOM_ENABLE() \
    [[LVIMSDK sharedInstance] getConfig].mIsChatroomEnabled
#endif //IS_LVIM_CHATROOM_ENABLE

//检测群组是否已启用
#ifndef IS_LVIM_GROUP_ENABLED
#define IS_LVIM_GROUP_ENABLED() \
    [[LVIMSDK sharedInstance] getConfig].mIsGroupEnabled
#endif //IS_LVIM_GROUP_ENABLED

//检测私信是否已启用
#ifndef IS_LVIM_PRIVATE_ENABLED
#define IS_LVIM_PRIVATE_ENABLED() \
    [[LVIMSDK sharedInstance] getConfig].mIsPrivateEnabled
#endif //IS_LVIM_PRIVATE_ENABLED

//检测是否是发布模式
#ifndef IS_LVIM_RELEASE_MODE
#define IS_LVIM_RELEASE_MODE() \
    ![[LVIMSDK sharedInstance] getConfig].mIsDebugMode
#endif //IS_LVIM_RELEASE_MODE

//----------------------------------------------------------------------------------------------------
//获取配置信息
#define GET_LVIM_SDK_CONFIG() \
    [[LVIMSDK sharedInstance] getConfig]
//获取媒体文件随机路径
#define GET_LVIM_MEDIA_CACHE_FILE_NAME(__fremote, __mftype, __fid, __tid, __fsuffix) \
    [[LVIMSDK sharedInstance] getCacheMediaFileName:__fremote mftype:__mftype fid:__fid tid:__tid fsuffix:__fsuffix];
//----------------------------------------------------------------------------------------------------

@class LVIMMsgManager;
@class LVIMGroupManager;
@class LVIMChatroomManager;
@class LVIMConversationManager;


@interface LVIMSDK : NSObject

/**
 @brief 消息管理
 */
@property (nonatomic,strong,readonly)LVIMMsgManager *msgManager;
/**
 @brief 群管理
 */
@property (nonatomic,strong,readonly)LVIMGroupManager *groupManager;
/**
 @brief 聊天室管理
 */
@property (nonatomic,strong,readonly)LVIMChatroomManager *chatroomManager;
/**
 @brief 会话管理
 */
@property (nonatomic,strong,readonly)LVIMConversationManager *conversationManager;

/**
 @brief 获取LVIM模块的实例
 **/
+(instancetype)sharedInstance;

//----------------------------------------------------------------------------------------------------

/**
 @brief 获取UUID
 **/
+(NSString*)getUUID;

/**
 @brief 获取当前的配置信息
 @return 未设置返回nil
 **/
-(LVIMConfig*)getConfig;

/**
 @brief 初始化
 @param appId 开放平台申请的appId
 @param secret 开放平台申请的secret
 **/
-(BOOL)initWithAppId:(NSString *)appId secret:(NSString *)secret;

/**
 @brief 设置LVIM的认证码
 @param uid 用户ID
 @param token 认证码(lvim)
 **/
-(void)setIMToken:(NSString*)uid token:(NSString*)token;

/**
 @brief 设置apns的推送token
 @param deviceToken didRegisterForRemoteNotificationsWithDeviceToken回调中解析后的deviceToken
 @param complete 上传完成回调，code为0是成功，其余都是失败
 @return 成功返回0, 否则返回非0
 **/
- (int)updatePushToken:(NSString *)deviceToken complete:(void(^)(int code, NSString *msg))complete;

/**
 @brief 设置是否允许LVIM记录连接的相关信息
 **/
-(void)setAllowRecordConnectionInfo:(BOOL)allow;

/**
 @brief 是否允许LVIM记录连接的相关信息
 **/
-(BOOL)allowRecordConnectionInfo;

/**
 @brief 设置直播间返回消息的处理代理(可根据需要设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)setChatroomReceiveMessageDelegate:(id<LVIMReceiveMessageDelegate>)delegate;

/**
 @brief 设置全局返回消息的处理代理(必须设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)setGlobalReceiveMessageDelegate:(id<LVIMReceiveMessageDelegate>)delegate;

/**
 @brief 设置消息刷新代理(可根据需要设置)
 @param interval 刷新间隔毫秒数
 @return 成功返回0, 否则返回非0
 **/
-(int)setMessageFlushDelegate:(id<LVIMReceiveMessageFlushDelegate>)delegate interval:(int32_t)interval;

/**
 @brief 设置模块事件代理(可根据需要设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)setModuleEventDelegate:(id<LVIMModuleEventDelegate>)delegate;

/**
 @brief 设置信息回报代理(可根据需要设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)setReportDelegate:(id<LVIMReportDelegate>)delegate;

/**
 @brief 设置上传日志的代理(可根据需要设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)setUploadLogFilterDelegate:(id<LVIMUploadLogFilterDelegate>)delegate;

/**
 @brief 设置设备id
 @param appDeviceID 设备id
 **/
-(int)setAppDeviceID:(NSString *)appDeviceID;

/**
 @brief 设置appXAID
 @param appXAID appXAID
 **/
-(int)setAppXAID:(NSString *)appXAID;

/**
 @brief 设置调试模式
 @param isEnable 是否开启调试模式
 **/
-(int)setDebugMode:(BOOL)isEnable;

/**
 @brief 设置日志可见状态
 @param isVisible 日志是否可见
 **/
-(int)setLogVisible:(BOOL)isVisible;

/**
 @brief 设置房间功能是否启用
 **/
-(int)setChatroomEnable:(BOOL)isEnable;

/**
 @brief 设置私信功能是否启用
 **/
-(int)setPrivateEnable:(BOOL)isEnable;

/**
 @brief 设置群组功能是否启用
 **/
-(int)setGroupEnable:(BOOL)isEnable;

//----------------------------------------------------------------------------------------------------

/**
 @brief 检测模块是否已经启动
 **/
-(BOOL)isStarted;

/**
 @brief 检测连接是否已经建立
 **/
-(BOOL)isConnected;

/**
 @brief 检测认证是否完成
 **/
-(BOOL)isAuthed;

/**
 @brief 检测是否登录成功
 **/
-(BOOL)isAppUserLoginSucceed;
//----------------------------------------------------------------------------------------------------

/**
 @brief 开始运行(调用之前必须正确完成配置设置)
 @return 成功返回0, 否则返回非0
 **/
-(int)start;

/**
 @brief 停止运行
 @param timeout 等待停止运行的最大超时毫秒数
 @return 成功停止返回1, 等待超时返回0, 否则返回<0的值
 **/
-(int)stop:(int)timeout;

/**
 @brief 用户登录
 @param uid 用户ID
 @param country 用户的国家代码
 @return 成功返回0, 否则返回非0
 **/
-(int)login:(NSString*)uid country:(NSString*)country loginType:(LVIM_LOGIN_TYPE)loginType;

/**
 @brief 用户登录
 @param uid 用户ID
 **/
-(int)login:(NSString*)uid loginType:(LVIM_LOGIN_TYPE)loginType;

/**
 @brief 用户登出
 @param waitFlushTimeout 等待数据同步超时毫秒数(小于等于0时不等待,默认1000)
 @param waitFinishTimeout 等待完成超时毫秒数(小于等于0时不等待,默认1000)
 @return 成功返回0, 否则返回非0
 **/
-(int)logout:(int)waitFlushTimeout waitFinishTimeout:(int)waitFinishTimeout;

/**
 @brief 用户登出
 @return 成功返回0, 否则返回非0
 **/
-(int)logout;

/**
 @brief 发送消息
 @param msg 消息内容
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)sendMessage:(LVIMMessage*)msg
          context:(id)context
         callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 用户私信黑名单管理
 @param uid 被拉黑的用户ID
 @param isAdd YES为添加,NO为删除
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)updatePrivateMessageBlackWithUid:(NSString *)uid
                                isAdd:(BOOL)isAdd
                               context:(id)context
                              callback:(LVIM_SEND_MESSAGE_CALLBACK)callback;

/**
 @brief 获取用户私信黑名单列表
 @param index 分页下标,第一页传入0
 例如,每页20条数据,第一页传0,第二页传20
 @param context 传入接收代理对象(可空)
 @param callback 结果回调(可空)
 @return 执行成功返回0, 否则返回非0
 **/
-(int)getPrivateMessageBlackWithIndex:(int)index
                               context:(id)context
                             callback:(LVIM_GROUP_ARRAY_CALLBACK)callback;

/**
 @brief 是否启用了http消息
 @return 未启用返回0, 独立连接启用返回1, 主连接启用返回2
 **/
-(int)isHttpMessageEnabled;

/**
 @brief 发送http业务消息
 @param path url请求地址后面的路径
 @param method 请求方法(GET/POST/PUT...)
 @param content 请求内容(可空)
 @param headers 附加的请求头部(可空)
 @param sendtmo 发送超时时间(1000毫秒 ~ 60000毫秒)
 @param recvtmo 接收超时时间(1000毫秒 ~ 60000毫秒)
 @param context 自定义上下文(可空)
 @param callback 自定义回调(可空)
 @return 成功返回0, 否则返回非0
 **/
-(int)sendHttpMessage:(NSString*)path
               method:(NSString*)method
              content:(NSString*)content
              headers:(NSString*)headers
              sendtmo:(int)sendtmo
              recvtmo:(int)recvtmo
              context:(id)context
             callback:(LVIM_SEND_HTTP_MESSAGE_CALLBACK)callback;
//error code 返回0表正常 1表示未登录
- (void)requestDebugToken:(void (^)(NSInteger errorCode, NSDictionary *responseDict))callback;

@end
