//
//  LVIMVideoMessage.h
//  CMIMLib
//
//  Created by 鹿容 on 2021/9/24.
//  Copyright © 2021 cmcm. All rights reserved.
//

#import "LVIMMediaMessage.h"
#import "LVIMFileContent.h"
#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@interface LVIMVideoMessage : LVIMMediaMessage

@property (nonatomic,strong,readonly)LVIMFileContent *videoFile;

@property (nonatomic,assign,readonly)CGSize videoSize;

@property (nonatomic,assign,readonly)float duration;

/**
 @brief 视频首帧图片,尺寸与videoSize相同
 **/
@property (nonatomic,strong,readonly)LVIMFileContent *imageFile;

@property (nonatomic,strong,readonly)UIImage *image;

@property (nonatomic,assign,readonly)CGSize imageSize;

/**
 @brief 视频首帧图片的缩略图
 **/
@property (nonatomic,strong,readonly)LVIMFileContent *thumbnailImageFile;

@property (nonatomic,strong,readonly)UIImage *thumbnailImage;

@property (nonatomic,assign,readonly)CGSize thumbnailImageSize;

/**
 @brief 构建视频私信请求
 @param fid 发送者ID(用户ID)
 @param tid 接收者ID(用户ID)
 @param type 消息类型别名(类型唯一)
 @param videoPath 视频路径,推荐传mp4文件
 @param pushContent 推送内容
 @return 成功返回0, 否则返回非0
 **/
-(int)buildVideoPrivateRequest:(NSString*)fid tid:(NSString*)tid type:(NSString*)type videoPath:(NSString *)videoPath content:(NSString *)content pushContent:(LVPushContent*)pushContent;

/**
 @brief 构建视频群组请求
 @param fid 发送者ID(用户ID)
 @param tid 接收者ID(用户ID)
 @param type 消息类型别名(类型唯一)
 @param videoPath 视频路径,推荐传mp4文件
 @param pushContent 推送内容
 @return 成功返回0, 否则返回非0
 **/
-(int)buildVideoGroupRequest:(NSString*)fid tid:(NSString*)tid type:(NSString*)type videoPath:(NSString *)videoPath content:(NSString *)content pushContent:(LVPushContent*)pushContent;

/**
 @brief 视频转码,默认转换成960x540的mp4文件
 **/
-(void)transVideoWithCompletionHandler:(nonnull void (^)(BOOL))handler;

/**
 @brief 下载视频首帧图片,下载成功后,会自动给image赋值
 **/
-(void)downloadImageWithProgress:(LVIM_PROGRESS_CALLBACK)progressCallback complete:(LVIM_DOWNLOAD_CALLBACK)downloadCallback;

/**
 @brief 下载缩略图,下载成功后,会自动给thumbnailImage赋值
 **/
-(void)downloadThumbnailImageWithProgress:(LVIM_PROGRESS_CALLBACK)progressCallback complete:(LVIM_DOWNLOAD_CALLBACK)downloadCallback;

/**
 @brief 下载视频,下载成功后,会自动给image赋值
 **/
-(void)downloadVideoWithProgress:(LVIM_PROGRESS_CALLBACK)progressCallback complete:(LVIM_DOWNLOAD_CALLBACK)downloadCallback;
@end

NS_ASSUME_NONNULL_END
