import 'dart:convert';

enum LVIMMsgState {
  sending,          // 发送中
  failed,           // 发送失败
  sent_succ_unread, // 发送成功，接收者未读
  sent_succ_read,   // 发送成功，接收者已读
  receive_unread,   // 接收成功，自己未读
  receive_read,     // 接收成功，自己已读
  encode_media_failed,//消息多媒体转码失败,暂时只有视频
  media_failed,       //消息上传多媒体文件失败
  readed_ack          //消息已读回执
}

class LVIMMsg {
  //消息来源
  int origin;

  //消息是否已读(目前只有客服接口有)
  int isReaded = 0;

  //消息类型
  int cmdType = -1;

  //内容类型
  int subType = -1;

  //来源ID(发送者ID)
  String fromID;

  //目标ID(接受者ID)
  String toID;

  //消息内容
  String msgContent;

  //扩展内容1(一般为msgType)
  String extend1;

  //扩展内容2
  String extend2;

  //扩展内容3
  String extend3;

  //扩展内容4(二进制)
  var extend4;

  //扩展内容5
  String extend5;

  //扩展内容6
  String extend6;

  //推送标题
  String pushTitle;

  //客户端消息ID
  int cmsgid;

  //服务端消息ID
  int smsgid;

  //消息序列号
  int sequence;

  //本地数据库ID
  int dbid;

  //客户端时间戳
  int ctime;

  //服务端时间戳
  int stime;

  //请求UUID
  String requestID;

  //临时本地路径(图片/音频等)
  String tempLocalPath;

  //HTTP相应代码
  int httpResponseCode;
  //服务响应代码
  int serviceResponseCode;
  //HTTP响应消息
  String responseContent;

  //HTTP请求地址
  String hpath;
  //HTTP请求耗时
  int hdelay;
  //服务端响应耗时
  int sdelay;
  //请求耗时
  int qdelay;

  // 消息发送状态
  LVIMMsgState sendState;



  void convertFromMap(dynamic args) {
    //消息来源
    this.origin = args["origin"];
    //消息来源
    this.isReaded = args["isReaded"];
    //消息类型
    this.cmdType = args["cmdType"];
    //内容类型
    this.subType = args["subType"];
    //来源ID(发送者ID)
    this.fromID = args["fromID"];
    //目标ID(接受者ID)
    this.toID = args["toID"];
    //消息内容
    this.msgContent = convertByteArrayToString(args["msgContent"]);
    //扩展内容1(一般为msgType)
    this.extend1 = convertByteArrayToString(args["extend1"]);
    //扩展内容2
    this.extend2 = convertByteArrayToString(args["extend2"]);
    //扩展内容3
    this.extend3 = convertByteArrayToString(args["extend3"]);
    //扩展内容4(二进制)
    this.extend4 = args["extend4"];
    //扩展内容5
    this.extend5 = convertByteArrayToString(args["extend5"]);
    //扩展内容6
    this.extend6 = convertByteArrayToString(args["extend6"]);
    //推送标题
    this.pushTitle = convertByteArrayToString(args["pushTitle"]);
    this.cmsgid = args["cmsgid"];
    this.smsgid = args["smsgid"];
    this.sequence = args["sequence"];
    this.dbid = args["dbid"];
    this.ctime = args["ctime"];
    this.stime = args["stime"];
    this.requestID = args["requestID"];
    this.tempLocalPath = args["tempLocalPath"];
    this.httpResponseCode = args["httpResponseCode"];
    this.serviceResponseCode = args["serviceResponseCode"];
    this.responseContent = args["responseContent"];
    this.hpath = args["hpath"];
    this.hdelay = args["hdelay"];
    this.sdelay = args["sdelay"];
    this.qdelay = args["qdelay"];
    if(args["sendState"] != null){
      this.sendState = LVIMMsgState.values[args["sendState"]];
    }
  }

  // 字节数组转String
  static String convertByteArrayToString(arg) {
    if (arg == null) {
      return null;
    }
    return utf8.decode(arg);
  }

  Map<dynamic, dynamic> toMap() => {
    "origin": origin == null ? null : origin,
    "isReaded": isReaded == null ? null : isReaded,
    "cmdType": cmdType == null ? null : cmdType,
    "subType": subType == null ? null : subType,
    "fromID": fromID == null ? null : fromID,
    "toID": toID == null ? null : toID,
    "msgContent": msgContent == null ? null : msgContent,
    "extend1": extend1 == null ? null : extend1,
    "extend2": extend2 == null ? null : extend2,
    "extend3": extend3 == null ? null : extend3,
    "extend4": extend4 == null ? null : extend4,
    "extend5": extend5 == null ? null : extend5,
    "extend6": extend6 == null ? null : extend6,
    "pushTitle": pushTitle == null ? null : pushTitle,
    "cmsgid": cmsgid == null ? null : cmsgid,
    "smsgid": smsgid == null ? null : smsgid,
    "sequence": sequence == null ? null : sequence,
    "dbid": dbid == null ? null : dbid,
    "ctime": ctime == null ? null : ctime,
    "stime": stime == null ? null : stime,
    "requestID": requestID == null ? null : requestID,
    "tempLocalPath": tempLocalPath == null ? null : tempLocalPath,
    "httpResponseCode": httpResponseCode == null ? null : httpResponseCode,
    "serviceResponseCode": serviceResponseCode == null ? null : serviceResponseCode,
    "responseContent": responseContent == null ? null : responseContent,
    "hpath": hpath == null ? null : hpath,
    "hdelay": hdelay == null ? null : hdelay,
    "sdelay": sdelay == null ? null : sdelay,
    "qdelay": qdelay == null ? null : qdelay,
    "sendState": sendState == null ? null : sendState.index,
  };

}